// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.MouseInputAdapter;
import javax.swing.text.JTextComponent;

import org.apache.log4j.Logger;

import de.uka.ilkd.key.gui.assistant.ProofAssistant;
import de.uka.ilkd.key.gui.assistant.ProofAssistantAI;
import de.uka.ilkd.key.gui.assistant.ProofAssistantController;
import de.uka.ilkd.key.gui.notification.NotificationManager;
import de.uka.ilkd.key.gui.notification.events.ExitKeYEvent;
import de.uka.ilkd.key.gui.notification.events.GeneralFailureEvent;
import de.uka.ilkd.key.gui.notification.events.GeneralInformationEvent;
import de.uka.ilkd.key.gui.notification.events.NotificationEvent;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.pp.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.proof.decproc.JavaDecisionProcedureTranslationFactory;
import de.uka.ilkd.key.proof.init.TacletSoundnessPOLoader;
import de.uka.ilkd.key.proof.mgt.BasicTask;
import de.uka.ilkd.key.proof.mgt.NonInterferenceCheck;
import de.uka.ilkd.key.proof.mgt.TaskTreeNode;
import de.uka.ilkd.key.rule.BuiltInRule;
import de.uka.ilkd.key.rule.BuiltInRuleApp;
import de.uka.ilkd.key.rule.ICSIntegerRule;
import de.uka.ilkd.key.rule.SimplifyIntegerRule;
import de.uka.ilkd.key.strategy.StrategyFactory;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.util.KeYResourceManager;
import de.uka.ilkd.key.util.ProgressMonitor;

public class Main extends JFrame {

//DO NOT CHANGE THE FOLLOWING 2 LINES, UNLESS YOU KNOW ABOUT PRCS-REPLACEMENT
    /* $Format: "    private static final String PRCSVERSION = \"$ProjectVersion$\";"$ */
    private static final String PRCSVERSION = "0.2032";
    
    private static final String VERSION="0.99 (internal: "+PRCSVERSION+")";
    private static final String COPYRIGHT="(C) Copyright 2001-2005 "
	+"Universit\u00e4t Karlsruhe, Universit\u00e4t Koblenz-Landau, "
	+"and Chalmers University of Technology";

    /**
     * The maximum number of recent files displayed.
     */
    private static final int MAX_RECENT_FILES = 8;
    /**
     * In which file to store the recent files.
     */
    private static final String RECENT_FILES_STORAGE = 
	System.getProperty("user.home")+
	 File.separator+".key"+File.separator+"recentFiles.props";

    /** Name of the config file controlling logging with log4j */
    private static final String LOGGER_CONFIGURATION = 
	System.getProperty("user.home")+
	 File.separator+".key"+File.separator+"logger.props";

    static {
	//@xxx preliminary: better store along with other settings.
	PresentationFeatures.ENABLED = true;
    }
    
    /** the tab bar at the left */
    private JTabbedPane	tabbedPane;

    /** the toolbar */
    private JToolBar toolBar;

    /** the current goal view */
    private JScrollPane goalView;

    /** the current proof tree*/
    private JScrollPane proofView;

    /** the list of current open goals*/
    private JScrollPane openGoalsView;

    /** the view of a sequent */
    private SequentView sequentView;

    /** the user constraint view */
    private UserConstraintView userConstraintView = null;

    /** the rule view */
    private RuleView ruleView = null;

    /** the strategy selection view */
    private StrategySelectionView strategySelectionView = null;

    /** the current user constraint */
    private ConstraintTableModel userConstraint = null;

    /** contains a list of all proofs */
    private JScrollPane proofListView;
    private TaskTree proofList;

    /** list of open goals of the current proof */
    private GoalList goalList;

    /** the mediator is stored here */
    protected KeYMediator mediator;
    
    /** the status line */
    MainStatusLine statusLine;
    
    /** the main progress monitor */
    protected ProgressMonitor progressMonitor = new MainProgressMonitor();
    
    /** listener to global proof events */
    private MainProofListener proofListener;

    /** listener to gui events */
    private MainGUIListener guiListener;
    private RecentFileMenu recentFiles;

    private boolean frozen = false;

    /** listener to changes of the user constraint */
    private MainConstraintTableListener constraintListener;

    private static KeYFileChooser fileChooser=null;
    
    private static final String PARA = 
       "<p style=\"font-family: lucida;font-size: 12pt;font-weight: bold\">";
    
    private static final String PARA_SMALL = 
        "<p style=\"font-family: lucida;font-size: 10pt;font-weight: bold\">";
     
    private JCheckBox resumeAutoMode;
    
    /** button for starting and stopping automatic mode */
    public static AutoModeAction autoModeAction;

    /** action for opening a KeY file */
    public static OpenFile openFileAction;

    /** action for saving a proof (attempt) */
    public static SaveFile saveFileAction;
    
    public static String AUTO_MODE_TEXT = "Start/stop automated proof search";

    /** if true then automaticaly start startAutoMode after the key-file is loaded*/
    public static boolean batchMode = false;
    
    /** if true then the proofs are loaded with brute force application of update simplifications*/
    public static boolean compatMode = false;

    /** if false then JML specifications are not parsed (useful for .key input) */
    public static boolean jmlSpecs = true;

    /** button to provide fast access to automatic mode configuration */
    private JButton autoModeConfigButton;

    public JButton reuseButton = new JButton("Reuse",IconFactory.reuseLogo());

    private JButton decisionProcedureButton;

    private JPopupMenu reusePopup = new JPopupMenu();

    protected static String fileNameOnStartUp = null;

    /** are we in stand-alone mode? (or with TCC?) */
    public static boolean standalone=  
        System.getProperty("key.together") == null;

    /** for locking of threads waiting for the prover to exit */
    public Object monitor = new Object();
    
    private static final String TACLET_OPTIONS_MENU_STRING = "ToolTip options ";
	
    protected static Main instance = null;

    /** menu for configuration of decision procedure */
    JMenu decisionProcedureOption = new JMenu("Decision Procedure Config");

    JRadioButtonMenuItem simplifyButton = new JRadioButtonMenuItem("Simplify", true);

    JRadioButtonMenuItem icsButton      = new JRadioButtonMenuItem("ICS", false);

    /** size of the tool bar icons */
    private int toolbarIconSize = 15;
    
    private ProverTaskListener taskListener;

    private NotificationManager notificationManager;


    /** creates prover -- private, use getInstance()
     * @param title the String containing the frame's title
     */ 
    protected Main(String title) {
	super(title);
	setIconImage(IconFactory.keyLogo());
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	configureLogger();
	proofListener      = new MainProofListener();
	guiListener        = new MainGUIListener();
	constraintListener = new MainConstraintTableListener();
        taskListener       = new MainTaskListener ();

        setMediator ( new KeYMediator ( this ) );

        initNotification();
        initProofList();	        
        layoutMain();
	initGoalList();
	initGUIProofTree();

	SwingUtilities.updateComponentTreeUI(this);
	ToolTipManager.sharedInstance().setDismissDelay(30000);
	setSize(1000,750);
	addWindowListener(new MainListener());
	
    }
    
    private void initNotification() {
        if (!batchMode) { 
            notificationManager = new NotificationManager(mediator);          
        }
    }
    
    public static Main getInstance() {
	return getInstance(true);
    }

    public static Main getInstance(boolean visible) {
        if (instance == null){
	    instance = new Main("KeY -- Prover");
	}
        if (!instance.isVisible()) instance.setVisible(visible); //XXX: enough?
        return instance;
    }
    
    
    public static void configureLogger() {
        if ((new File(LOGGER_CONFIGURATION)).exists())
            org.apache.log4j.PropertyConfigurator.configureAndWatch(
		LOGGER_CONFIGURATION, 1500);
        else {
	    org.apache.log4j.BasicConfigurator.configure();
	    Logger.getRootLogger().setLevel(org.apache.log4j.Level.ERROR);
	}
    }

    public String getPrcsVersion()
    {
        return PRCSVERSION;
    }

    public void updateUI() {
	if (goalView != null) goalView.updateUI();
	if (proofView != null) proofView.updateUI();
	if (openGoalsView != null) openGoalsView.updateUI();
	if (userConstraintView != null) userConstraintView.updateUI();
	if (ruleView != null) ruleView.updateUI();
    	if (proofListView != null) proofListView.updateUI();
    }

    /** sets the mediator to corresspond with other gui elements
     * @param m the KeYMediator
     */
    private void setMediator(KeYMediator m) {
	if (mediator !=  null) unregister();
	mediator = m;
	mediator.setSimplifier(
            ProofSettings.DEFAULT_SETTINGS.getSimultaneousUpdateSimplifierSettings().getSimplifier());

	// The following needs to be called before the SequentView
	// is created.
	Config.DEFAULT.setDefaultFonts();
	sequentView = new SequentView(mediator);
	register();
    }

    /** register several listeners */
    private void register() {	
	mediator.addKeYSelectionListener(proofListener);
	mediator.addAutoModeListener(proofListener);
	mediator.addGUIListener(guiListener);
    }

    /** unregister several listeners */
    private void unregister() {
	mediator.removeKeYSelectionListener(proofListener);
	mediator.removeAutoModeListener(proofListener);
	mediator.removeGUIListener(guiListener);	
    }
   

    /** return the mediator
     * @return the mediator
     */
    public KeYMediator mediator() {
	return instance.mediator;
    }

    /** paints empty view */
    private void paintEmptyViewComponent(JScrollPane pane, String name) { 	
	pane.setBorder(new TitledBorder(name));
	pane.setBackground(Color.white);
	pane.getViewport().setBackground(Color.white);	
	pane.setMinimumSize(new java.awt.Dimension(150,0));
    }

    /** adds a new tab */
    public void addTab(String tabname, JScrollPane view, String description) {
	tabbedPane.addTab(tabname, null, view, description);
    }

    /** sets the tool bar */
    private void setToolBar(JToolBar bar) {
	toolBar = bar;	
	toolBar.setFloatable(true);
	toolBar.setRollover(true);
    }

    /** lays out the main frame */
    protected void layoutMain() {
	// set overall layout manager
	getContentPane().setLayout(new BorderLayout());	

	// create empty components first
	setJMenuBar(new JMenuBar());
	setToolBar(new JToolBar("Proof Control"));
	
        autoModeAction = new AutoModeAction();
	openFileAction = new OpenFile();
	saveFileAction = new SaveFile();

	// ============================================================
	// ==================  create empty views =====================
	// ============================================================
	
	goalView = new JScrollPane();	
	paintEmptyViewComponent(goalView, "Current Goal");	

	proofView = new JScrollPane();
	paintEmptyViewComponent(proofView, "Proof");

	openGoalsView = new JScrollPane();
	paintEmptyViewComponent(openGoalsView, "Open Goals");

	userConstraintView = new UserConstraintView ();
	if ( mediator != null ) {
	    userConstraintView.setMediator(mediator);
	}

	strategySelectionView = new StrategySelectionView();
	if ( mediator != null ) {
	    strategySelectionView.setMediator(mediator);
	}

	ruleView = new RuleView ();
	if ( mediator != null ) {
	    ruleView.setMediator(mediator);
	}

	// MENU BAR
	// ============================================================
	// ==================  create menu bar entries ================
	// ============================================================
	createMenuBarEntries();

	// TOOL BAR SECTION 
	// ============================================================
	// ==================== create tool bar entries ===============
	// ============================================================

	toolBar.add(createAutoModeButton());
        toolBar.addSeparator();                        
	toolBar.add(createAutoModeConfigButton());
	toolBar.addSeparator();
	toolBar.add(createResumeAutoModeCheckBox());
	toolBar.addSeparator();
	toolBar.add(createDecisionProcedureButton());
	toolBar.addSeparator();

        JButton goalBackButton = new JButton("Goal Back");
	goalBackButton.setIcon(IconFactory.goalBackLogo(toolbarIconSize));	
	goalBackButton.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
	       setBack();
	   }
        });

	
        toolBar.add(goalBackButton);	
        toolBar.addSeparator();
        toolBar.add(reuseButton);
        reuseButton.setEnabled(false);
        reuseButton.setToolTipText("Start proof reuse (when template available)");
	JMenuItem singleStepReuse = new JCheckBoxMenuItem("Single step");
	singleStepReuse.setSelected(false);
	singleStepReuse.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                mediator.setContinuousReuse(
                    !((JCheckBoxMenuItem)e.getSource()).isSelected());
	    }
        });
        reusePopup.add(singleStepReuse);
        reuseButton.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    reusePopup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        });
        toolBar.addSeparator();                        

	JToolBar fileOperations = new JToolBar("File Operations");
        fileOperations.add(createOpenFile());
        fileOperations.add(createOpenMostRecentFile());
	fileOperations.add(createSaveFile());

	goalView.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW ).put(
           KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK), 
           "copy");
        goalView.getActionMap().put("copy", new AbstractAction() {
           public void actionPerformed(ActionEvent e) {
              copyHighlightToClipboard(sequentView);
           }
        });
                
	JPanel toolBarPanel = new JPanel();
	toolBarPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
	toolBarPanel.add(toolBar);
	toolBarPanel.add(fileOperations);

        getContentPane().add(toolBarPanel, BorderLayout.PAGE_START);
        
	// ============================================================
	// ==================== create tabbed pane ====================
	// ============================================================

	tabbedPane = new JTabbedPane();

	addTab("Proof", proofView, 
	       "The current state of the "
	       +"proof as tree");

	addTab("Goals", openGoalsView, 
	       "The currently open goals");

	tabbedPane.addTab("User Constraint", null, userConstraintView, 
			  "Currently chosen metavariable instantiations");

	tabbedPane.addTab("Proof Search Strategy", null, strategySelectionView, 
			  "Select strategy for automated proof search");

	tabbedPane.addTab("Rules", null, new JScrollPane(ruleView), 
			  "All available rules");
	tabbedPane.setSelectedIndex(0);
	tabbedPane.setPreferredSize ( new java.awt.Dimension ( 250, 440 ) );

	proofListView.setPreferredSize ( new java.awt.Dimension ( 250, 100 ) );
	paintEmptyViewComponent(proofListView, "Tasks");

	JSplitPane leftPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
					     proofListView, tabbedPane) {
            public void setUI(javax.swing.plaf.SplitPaneUI ui) {
                try{ super.setUI(ui); } catch(NullPointerException e) 
		    {Debug.out("Exception thrown by class Main at setUI"); }
            }
        }; // work around bug in 
        // com.togethersoft.util.ui.plaf.metal.OIMetalSplitPaneUI

        leftPane.setOneTouchExpandable(true);
        
	JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					      leftPane, goalView) {
            public void setUI(javax.swing.plaf.SplitPaneUI ui) {
                try{ super.setUI(ui); } catch(NullPointerException e) 
		    { Debug.out("Exception thrown by class Main at setUI");}
            }
        }; // work around bug in 
        // com.togethersoft.util.ui.plaf.metal.OIMetalSplitPaneUI
	splitPane.setResizeWeight(0); // the right pane is more important
	splitPane.setOneTouchExpandable(true);
	getContentPane().add(splitPane, BorderLayout.CENTER);
	 

	statusLine = new MainStatusLine
	    ("<html>"+PARA+COPYRIGHT+
	     PARA+
	     "KeY is free software and comes with ABSOLUTELY NO WARRANTY."+
	     " See Help | License.",
	     getFont () );
        getContentPane().add(statusLine, BorderLayout.SOUTH);
        


// *********************** UGLY INSPECTION CODE **********************
	goalView.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW ).put(
           KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK), 
           "show_tree");
        goalView.getActionMap().put("show_tree", new AbstractAction() {
           public void actionPerformed(ActionEvent e) {
              System.err.println(sequentView.getMousePosInSequent().
                 getPosInOccurrence().posInTerm());

              Term t = 
                 sequentView.getMousePosInSequent().getPosInOccurrence().subTerm();
System.err.println("****************** "+t.op().getClass());
System.err.println(t.hashCode());
              t.tree();
              
//              if (t instanceof ProgramTerm) {
                 de.uka.ilkd.key.java.visitor.JavaASTWalker w = 
                    new de.uka.ilkd.key.java.visitor.JavaASTWalker(
                       t.javaBlock().program()) {
                          protected void walk(ProgramElement node) {
	                      if (node != root()) doAction(node);
	                      if (node instanceof NonTerminalProgramElement) {
	                          NonTerminalProgramElement nonTerminalNode = 
		                      (NonTerminalProgramElement) node;
	                          for (int i = 0; 
                                       i<nonTerminalNode.getChildCount(); 
                                       i++) {
		                      walk(nonTerminalNode.getChildAt(i));
	                          }	    
	                      }
                          }
                    
                    protected void doAction(ProgramElement node) {
                       if (node instanceof Statement &&
                           !(node instanceof StatementBlock))
                          System.err.println(node.getClass()+":- "+node);
                          if (node instanceof 
                              de.uka.ilkd.key.java.statement.MethodFrame)
                              System.err.println(
                          ((de.uka.ilkd.key.java.statement.MethodFrame)node).
                                                 getExecutionContext());
                    }
                 };
                 w.start();
//              }

           }
        });


	goalView.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW ).put(
           KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK), 
           "show_reuse_state");
        goalView.getActionMap().put("show_reuse_state", new AbstractAction() {
           public void actionPerformed(ActionEvent e) {
               mediator().showReuseState();
           }
        });

	goalView.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW ).put(
           KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, ActionEvent.ALT_MASK),
           "show_reuse_cntd");
        goalView.getActionMap().put("show_reuse_cntd", new AbstractAction() {
           public void actionPerformed(ActionEvent e) {
               mediator().showPreImage();
           }
        });



// **************************************************************************        
    }

    

    /**
     * Create popup menu for strategy selection.
     * A proof must be loaded.
     * @return strategy selection menu
     */
    private JPopupMenu createStrategyMenu () {
        JPopupMenu menu = new JPopupMenu ( "Strategies" );
        
        addStrategySelectionItems ( menu );
        
        menu.addSeparator();
        
        addMaxRuleAppsSlider ( menu );
        
        return menu;
    }

    /**
     * Add slider for choosing max. rule application count to popup menu.
     * The slider is "almost" logarithmic: it features equidistant ticks
     * corresponding to values 1, 2, 3, ..., 9, 10, 20, 30, ..., 90, 100, ...
     * @param menu
     */
    private void addMaxRuleAppsSlider ( JPopupMenu menu ) {
        int maxRuleApps = mediator().getMaxAutomaticSteps();

	final JLabel numLabel = 
	    new JLabel ( "Max. rule applications: "+maxRuleApps );
    
	// determine initial position of slider
        int initPos = 0;
        if ( maxRuleApps > 0 ) {
            int major = (int)(Math.log ( maxRuleApps ) / Math.log ( 10 ));
            int minor = maxRuleApps / (int)Math.pow(10, major) - 1;
            initPos = Math.max ( 0, Math.min ( MAX_RULE_APPS_LOG10*9,
                                               major * 9 + minor ));
        }
        
        final JSlider slider = new JSlider(JSlider.HORIZONTAL,
                0, MAX_RULE_APPS_LOG10*9, initPos);
        
        // set up slider labels
        Hashtable labelTable = new Hashtable();
       
        for ( int n = 0; n <= MAX_RULE_APPS_LOG10; n++ ) {
            int val = (int)Math.pow(10, n);
            labelTable.put ( new Integer ( n*9 ), 
                             new JLabel ( ""+val ) );
        }
        
        slider.setLabelTable ( labelTable );
        slider.setPaintLabels ( true );

        // show ticks
        slider.setMajorTickSpacing ( 9 );
        slider.setMinorTickSpacing ( 1 );
        slider.setPaintTicks ( true );
        
        // add change listener
        slider.addChangeListener( new ChangeListener() {
            public void stateChanged ( ChangeEvent e ) {
                int n = slider.getValue();
                int major = n / 9;
                int minor = n % 9;
                int val = (minor+1)*(int)Math.pow(10, major);
                numLabel.setText( "Max. rule applications: "+val );
                mediator().setMaxAutomaticSteps ( val );
            }
        });
        
	menu.add ( numLabel );
        menu.add ( slider );
    }

    /**
     * Add items for selecting a strategy to popup menu.
     * @param menu
     */
    private void addStrategySelectionItems ( JPopupMenu menu ) {
        // get sorted array of strategy names
        final Namespace strategyNamespace =
            StrategyFactory.getStrategyNamespace();
        Named[] strategies = strategyNamespace.elements().toArray();
        Arrays.sort(strategies, new Comparator () {
            public int compare ( Object a, Object b ) {
                return ((Named)a).toString().compareTo(((Named)b).toString());
            }
        });
        
        
        // get name of active strategy
        Name activeStrategy = mediator().getStrategy(); 
        
        ButtonGroup group = new ButtonGroup ();
        
        // add all strategies to popup menu
        for ( int n = 0; n < strategies.length; n++ ) {
            final Name name = strategies[n].name ();
            // strategy is selected if its name matches that 
            // of the active strategy
            final boolean selected = name.equals ( activeStrategy );
            
            final JRadioButtonMenuItem item = 
                new JRadioButtonMenuItem ( name.toString (), selected );

            // reselecting the active strategy will do nothing
            if ( !selected ) {
                item.addActionListener(new ActionListener() {
                    public void actionPerformed ( ActionEvent e ) {
                        mediator.setStrategy(name);
                    }
                });
            }
            
            menu.add ( item );
            group.add ( item );
        }
    }
    
    private JButton createAutoModeButton () {
        return new JButton (autoModeAction);
    }

    private JComponent createOpenMostRecentFile() {
        final JButton button = new JButton();
        button.setAction(new OpenMostRecentFile());              
        return button;
    }

    private JComponent createOpenFile() {	
        final JButton button = new JButton();	
        button.setAction(openFileAction);              
	button.setText(null);
        return button;
    }


    private JComponent createSaveFile() {	
        final JButton button = new JButton();	
        button.setAction(saveFileAction);              
	button.setText(null);
        return button;
    }
    

    private JComponent createAutoModeConfigButton () {
        autoModeConfigButton = new JButton();
        
        autoModeConfigButton.setText("(no strategy)");
	autoModeConfigButton.setEnabled( false );
        autoModeConfigButton.setHorizontalTextPosition(SwingConstants.LEADING);

	final Icon arrowIcon = 
            IconFactory.selectStrategyArrow(toolbarIconSize);
	    autoModeConfigButton.setIcon( arrowIcon );

        autoModeConfigButton.addActionListener( new ActionListener () {
            public void actionPerformed ( ActionEvent e ) {
                if ( mediator().ensureProofLoaded() ) {
                    Component c = autoModeConfigButton;
                    createStrategyMenu()
                        .show( c, 0, c.getHeight() );
                }
            }
        });
        
        return autoModeConfigButton;
    }
    
    /**
     * Updates label of the automode button.
     * Updates the label of the automode button after the 
     * proof or strategy has changed.
     */
    private synchronized void updateAutoModeConfigButton () {
        String label = "(no strategy)";
	autoModeConfigButton.setEnabled( false ); 

        final Proof proof = mediator ().getProof();
        if ( proof != null ) {
            label = mediator ().getStrategy ().toString ();
            label = getShortenedLabel ( label );
	    autoModeConfigButton.setEnabled( true ); 
        }
        autoModeConfigButton.setText( label );
        autoModeConfigButton.
        setMaximumSize(autoModeConfigButton.getPreferredSize());
    }
    
    /**
     * Get a shortened version of a button label.
     * @param label
     * @return <code>label</code>, or an alternate version 
     * of <code>label</code> with reduced width 
     */
    private String getShortenedLabel ( String label ) {
        String rv = label;
        if ( label.length() > 20 ) {
            int length = label.length();
            int index = length / 2;
            int n = nearestIndexOf ( label, ' ', index );
            if ( n >= 0 ) {
                rv = "<HTML>"+PARA_SMALL+label.substring(0, n)
                    +PARA_SMALL+label.substring(n, length);
            }
        }
        return rv;
    }

    /**
     * Returns the index of the nearest occurrence of the
     * specified character, searching both ways from the
     * specified index.
     * @param s string to search
     * @param ch character to search for
     * @param indexFrom index where the search starts
     * @return nearest index of <code>ch</code>, or -1 if
     * no occurrence is found.
     */
    private int nearestIndexOf ( String s, char ch, int indexFrom ) {
        int n1 = s.lastIndexOf( ch, indexFrom);
        int n2 = s.indexOf( ch, indexFrom);
        int n = -1;
        if ( n1 >= 0 && (n1 > s.length() - n2 || n2 == -1) ) {
            n = n1;
        } else if ( n2 >= 0 ) {
            n = n2;
        }
        return n;
    }

    /** 
     * creates the toolbar button invoking decision procedures like
     * ICS, Simplify 
     */
    private JButton createDecisionProcedureButton() {
	String toolTipText = 
	    "Run " + ProofSettings.DEFAULT_SETTINGS.
	    getDecisionProcedureSettings().getDecisionProcedure();

	decisionProcedureButton = new JButton();
	decisionProcedureButton.setToolTipText(toolTipText);
	decisionProcedureButton.setText(toolTipText);
	
	
	// select icon 
	if (ProofSettings.DEFAULT_SETTINGS.
	    getDecisionProcedureSettings().useSimplify()) {

	    decisionProcedureButton.setIcon
		(IconFactory.simplifyLogo(toolbarIconSize));

	} else if (ProofSettings.DEFAULT_SETTINGS.
		   getDecisionProcedureSettings().useICS()) { 
	    decisionProcedureButton.setIcon
		(IconFactory.icsLogo(toolbarIconSize));
	}
	
        decisionProcedureButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mediator.resetNrGoalsClosedByHeuristics();
                
                // TODO: separate from GUI?
                try {
                    // an exception is thrown if a binary for a built-in
                    // rule is missing or the user has no execution
                    // permission
                    applySimplificationOnGoals();
                } catch (RuntimeException re) {
                    mediator.notify(new GeneralFailureEvent(re.getMessage()));
                }
                
                int nrGoalsClosed = mediator.getNrGoalsClosedByAutoMode();             
                
                if (nrGoalsClosed > 0 && !mediator.getProof().closed()) {
                    final String informationMsg =
                        nrGoalsClosed + ((nrGoalsClosed > 1) ? 
                            " goals have been closed": " goal has been closed");
                    mediator.notify
                    (new GeneralInformationEvent(informationMsg));			   
                }
            }
        });		

	return decisionProcedureButton;
    }

    /** 
     * create the resume automode checkbox
     */
    private JCheckBox createResumeAutoModeCheckBox() {
	resumeAutoMode = new JCheckBox("<html>Autoresume</html>");		
	resumeAutoMode.setToolTipText("Restarts the strategy after an interactive proof step.");
	resumeAutoMode.setBorderPaintedFlat(true);
	
	resumeAutoMode.setIcon
	    (IconFactory.resumeDisabledLogo(toolbarIconSize));		
	
	resumeAutoMode.setMaximumSize(resumeAutoMode.getPreferredSize());
	resumeAutoMode.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
		    if (e.getStateChange() == ItemEvent.SELECTED) {
			resumeAutoMode.
			    setIcon(IconFactory.resumeLogo(toolbarIconSize));
			mediator().setResumeAutoMode(true);
		    } 
		    else if (e.getStateChange() == ItemEvent.DESELECTED) {
			resumeAutoMode.
			    setIcon(IconFactory.
				    resumeDisabledLogo(toolbarIconSize));
			mediator().setResumeAutoMode(false);
		    }
		    else Debug.out("Automode checkbox undefined state: "+
				   e.getStateChange());
		    
	       }
	    });
	return resumeAutoMode;
    }
    
    public ProverTaskListener getProverTaskListener() {
        return taskListener;
    }

    /**
     * @return the status line object
     */
    public MainStatusLine getStatusLine () {
	return statusLine;
    }

    private void setStandardStatusLineImmediately () {
	statusLine.reset ();
    }

    /**
     * Make the status line display a standard message, make progress
     * bar and abort button invisible
     */
    public void setStandardStatusLine() {
	if ( SwingUtilities.isEventDispatchThread () )
	    setStandardStatusLineImmediately ();
	else {
	    Runnable lineUpdater = new Runnable() {
		    public void run() {
			setStandardStatusLineImmediately ();
		    }
		};
	    SwingUtilities.invokeLater(lineUpdater);
	}
    }

    private void setStatusLineImmediately(String s) {
	statusLine.reset ();
	statusLine.setStatusText(s);
	statusLine.setProgressPanelVisible(false);
    }

    private void setStatusLineImmediately(String s, int totalChars) {
	statusLine.reset ();
	statusLine.setStatusText(s);
	getProgressMonitor().setMaximum(totalChars);
	statusLine.setProgressPanelVisible(true);
	//statusLine.setAbortButtonEnabled(false);
    }

    /**
     * Display the given message in the status line, make progress bar
     * and abort button visible, set the progress bar range to the
     * given value, set the current progress to zero
     */    
    public void setStatusLine(String s, int totalChars) {
	if ( SwingUtilities.isEventDispatchThread () )
	    setStatusLineImmediately ( s, totalChars );
	else {
	    final String str = s;
	    final int max = totalChars;
	    Runnable lineUpdater = new Runnable() {
		    public void run() {
			setStatusLineImmediately ( str, max );
		    }
		};
	    SwingUtilities.invokeLater(lineUpdater);
	}
    }

    /**
     * Display the given message in the status line, make progress bar
     * and abort button invisible
     */    
    public void setStatusLine(String s) {
	if ( SwingUtilities.isEventDispatchThread () )
	    setStatusLineImmediately ( s );
	else {
	    final String str = s;
	    Runnable lineUpdater = new Runnable() {
		    public void run() {
			setStatusLineImmediately ( str );
		    }
		};
	    SwingUtilities.invokeLater(lineUpdater);
	}
    }


    /** Get the progress monitor that will update a progress bar in 
     * a corner of the main window. */
    public ProgressMonitor getProgressMonitor() {
	return progressMonitor;
    }

    /**
     * Freeze the main window by blocking all input events, except
     * those for the status line (i.e. the abort button within the
     * status line)
     */
    public void freezeExceptAutoModeButton () {
	if ( !frozen ) {
	    frozen = true;

	    Component glassPane = new BlockingGlassPane ( getContentPane () );
	    setGlassPane ( glassPane );
	    glassPane.setVisible ( true );
	}
    }

    public void unfreezeExceptAutoModeButton () {
	if ( frozen ) {
	    getGlassPane ().setVisible ( false );
	    frozen = false;
	}
    }

    /** saves settings */
    private void saveSettings() {
	ProofSettings.DEFAULT_SETTINGS.saveSettings();
	recentFiles.store(RECENT_FILES_STORAGE);
    }

    /** exit  */
    protected void exitMain() {
	boolean quit = false;
	if (ProofSettings.DEFAULT_SETTINGS.changed()) {
	    final int option = JOptionPane.showOptionDialog
		(Main.this, "Settings have changed. "+
		 "Should they be saved before quitting?", 
		 "Exit", 
		 JOptionPane.YES_NO_CANCEL_OPTION,
		 JOptionPane.QUESTION_MESSAGE,
		 null,
		 new String[]{"Save and Quit", 
			      "Quit, don't Save", 
			      "Cancel"},
		 null);   
	    if (option == 0) {
		saveSettings();
		quit = true;
	    } else if (option == 1) {
		// do not save but quit
		quit = true;
	    }
	} else {
	    final int option = JOptionPane.showConfirmDialog
		(Main.this, "Really Quit?", "Exit", 
		 JOptionPane.YES_NO_OPTION);		   	    
	    if (option == JOptionPane.YES_OPTION) {
		quit = true;
	    } 
	}

	recentFiles.store(RECENT_FILES_STORAGE);

	if (quit) {            
	    mediator.fireShutDown(new GUIEvent(this));

            if (standalone) {
		//wait some seconds; give notification sound a bit time
		try {
		    Thread.sleep(1000);
		} catch(InterruptedException ie) {
		    Debug.out("Thread has been interrupted.");
		}
	        System.out.println("Have a nice day.");
    	        System.exit(-1);
            }
	}
	//Release threads waiting for the prover to exit
	synchronized (this.monitor) {
	    this.monitor.notifyAll();
	}
    }

    /** opens selection dialog for strategy */
    protected void selectStrategy() {
        if (mediator().ensureProofLoaded()) {
            new StrategySelector(this, mediator());
        }
    }

    /** opens selection dialog for the maximum tooltip line number */
    protected void selectMaxTooltipLines() {
       	ViewSelector vselector = new ViewSelector(this);
        vselector.setVisible(true);
    }


    /** opens selection dialog for choices */
    protected void selectChoices() {
	ChoiceSelector selector = new ChoiceSelector
	    (ProofSettings.DEFAULT_SETTINGS.getChoiceSettings());
    }

    public void showSpecBrowser(){
	if(mediator.getProof() == null){
	    mediator.notify
	    (new GeneralFailureEvent("Please load a proof first"));
	}else{
	    JMLSpecBrowser specBrowser = 
		JMLSpecBrowser.getJMLSpecBrowser(mediator);
	}
    }

    protected void showActivatedChoices(){
        Proof currentProof = mediator.getProof();
	if(currentProof == null){
	    mediator.notify
	    (new GeneralInformationEvent
	            ("No Options available.", 
	                    "If you wish to see Taclet Options " +
	            "for a proof you have to load one first"));
	}else{
	    Iterator it = currentProof.getSettings().getChoiceSettings()
                    .getDefaultChoices().values().iterator();
	    String message = "Active Taclet Options:\n";
	    while(it.hasNext()){
		message += it.next().toString() + "\n";
	    }
	    final JTextComponent activeOptions = new JTextArea(message);
	    activeOptions.setEditable(false);
	    JOptionPane.showMessageDialog
		(Main.this, activeOptions,
		 "Active Taclet Options",
		 JOptionPane.INFORMATION_MESSAGE);
	}
    }
    
    /** 
     * for debugging - opens a window with the settings from current Proof and
     * the default settings
     */
    protected void showSettings(){
        String message;

        message = "Default Settings: \n" + 
	    ProofSettings.DEFAULT_SETTINGS.settingsToString() + "\n----------\n";
        message += "Settings[CurrentProof]:\n" + 
	    ((mediator.getProof()==null)?"No proof loaded." : 
	     mediator.getProof().getSettings().settingsToString());
        
        final JTextArea settings = new JTextArea(message, 30, 80);
        settings.setEditable(false);
        settings.setLineWrap(true);
        
        JScrollPane settingsPane = new JScrollPane(settings);
        
        JOptionPane.showMessageDialog(Main.this, settingsPane,
                "Settings", JOptionPane.INFORMATION_MESSAGE);
    }
    

    /** opens configuration dialog for the simultaneous update simplifier */
    protected void configSimultaneousUpdateSimplifier() {
	SimultaneousUpdateSimplifierConfiguration config = 
	    new SimultaneousUpdateSimplifierConfiguration
	    (mediator(), 
	     ProofSettings.DEFAULT_SETTINGS.getSimultaneousUpdateSimplifierSettings());
    }
    
    /** opens a configuration dialog for the 
     *  loaded libraries 
     **/
    protected void configLibraries() {
        LibrariesConfiguration config = 
            new LibrariesConfiguration
            (mediator(), 
             ProofSettings.DEFAULT_SETTINGS.getLibrariesSettings());
    }

    protected void setBack() {
       mediator.setBack();
    }

    protected void makePrettyView() {
	if (mediator().ensureProofLoadedSilent()) {
	    if (!PresentationFeatures.ENABLED) {
		mediator().getNotationInfo().setBackToDefault();
	    } else {	    
		PresentationFeatures.modifyNotationInfo(mediator.getNotationInfo(),
							mediator.func_ns());
	    }
	    mediator().getSelectedProof().updateProof();
	    userConstraintView.updateTableDisplay (); // %%% HACK
	}

    }

    public JMLPOAndSpecProvider getJMLPOAndSpecProvider(){
       return new JMLEclipseAdapter(mediator);
    }

    public void showLicense() {
			     
	URL lic = 
	KeYResourceManager.getManager().getResourceFile(Main.class,
							"LICENSE.TXT"); 
	StringBuffer sb=new StringBuffer();
	try {
	    FileInputStream inp=new FileInputStream(lic.getFile());
	    while (inp.available()>0) sb.append((char)inp.read());	   
	} catch (IOException ioe) {
	    System.out.println("License file cannot be loaded or is missing: \n"+
			       COPYRIGHT+"\nKeY is protected by the "
			       +"GNU General Public License");
	    sb=new StringBuffer(COPYRIGHT+"\nKeY is protected by the "
				+"GNU General Public License");
	} 
	String s=sb.toString();
	JScrollPane scroll = new JScrollPane();
	JTextArea text = new JTextArea(s,20,40);
	text.setEditable(false);
	text.setCaretPosition(0);
	scroll.setViewportView(text);
	JFrame fr = new JFrame("KeY License");
	fr.getContentPane().setLayout(new BorderLayout());
	fr.getContentPane().add(scroll,BorderLayout.CENTER);
	JButton ok = new JButton("OK");
	ok.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {		   
		    ((JFrame)((JButton)e.getSource())
		     .getTopLevelAncestor()).dispose();
		}});
	fr.getContentPane().add(ok, BorderLayout.SOUTH);
	fr.setSize(600,900);
	fr.getContentPane().add(scroll);
	fr.setVisible(true);
    }

    public void showAbout() {
	String aspects = compiledAspects();
	JOptionPane pane =new JOptionPane(
            COPYRIGHT+"\n\nWWW: http://key-project.org\n\nVersion "+
            VERSION
	    + ((aspects.length()==0) ? "":
	       "\nCompiled with Aspects:\n"+aspects)
	    ,
	    JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION,
	    IconFactory.keyLogo(105,60));
	JDialog dialog = pane.createDialog(this, "The KeY Project");
	dialog.setVisible(true);
    }
    
    
    /** Return a list of aspects compiled into the system,
     * one by line.  The idea is that the aspects will advise
     * this method to add themselves to the list. */
    public String compiledAspects() {
	return "";
    }

    /** registers a new menuitem at the given menu
     * @param menu the JMenu where to register the new item
     * @param item the JMenuItem to register
     */
    public void registerAtMenu(JMenu menu, JMenuItem item) {
	menu.add(item);
    }

   /** adds a separator to the given menu
     */
    public void addSeparator(JMenu menu) {	
	menu.addSeparator();
    }


    /** the number of goals the goal list currently contains */
    public int displayedOpenGoalNumber () {
        return goalList.getModel ().getSize ();
    }
    
    
    /** starts the goal choice frame */
    protected void initGoalList() {
	goalList = new GoalList(mediator);
	goalList.setSize(goalList.getPreferredSize());
	openGoalsView.setViewportView(goalList);
    } 
    
    protected void initProofList() {
        proofList = new TaskTree(mediator);
        proofListView = new JScrollPane();	
    }

    protected void addToProofList(de.uka.ilkd.key.proof.ProofAggregate plist) {
        proofList.addProof(plist);	
	// GUI 
	proofList.setSize(proofList.getPreferredSize());
	proofListView.setViewportView(proofList);
    }


    /** starts the gui proof tree frame */
    protected void initGUIProofTree() {
	ProofTreeView guiProofTree = new ProofTreeView(mediator);
	guiProofTree.setSize(guiProofTree.getPreferredSize());
	guiProofTree.setVisible(true);
	proofView.setViewportView(guiProofTree);
    }
    
    static JFrame clipBoard = null;
    static java.awt.TextArea clipBoardTextArea;
    
    public static void copyHighlightToClipboard(SequentView view) {
       if (clipBoard == null) {
          clipBoard = new JFrame("KeY Clipboard");
          clipBoardTextArea = new java.awt.TextArea();
          clipBoard.getContentPane().add(clipBoardTextArea);
       }
       String s = view.getHighlightedText();
       // now CLIPBOARD
       java.awt.datatransfer.StringSelection ss = 
          new java.awt.datatransfer.StringSelection(s);
       clipBoard.getToolkit().getSystemClipboard().setContents(ss,ss);
       // now PRIMARY
       clipBoardTextArea.setText(s);
       clipBoard.pack();
       clipBoard.setState(java.awt.Frame.ICONIFIED);
       clipBoard.setVisible(true);
       clipBoardTextArea.selectAll();
    }
    
    

    protected JMenu createFileMenu() {
	JMenu fileMenu = new JMenu("File");
	fileMenu.setMnemonic(KeyEvent.VK_F);

	JMenuItem load = new JMenuItem();
	load.setAction(openFileAction);

	JMenuItem save = new JMenuItem();
	save.setAction(saveFileAction);


	registerAtMenu(fileMenu, load);
	registerAtMenu(fileMenu, save);


	JMenuItem tacletPOItem = new JMenuItem ( "Load Non-Axiom Lemma ..." );
	tacletPOItem.addActionListener ( new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                if ( mediator().ensureProofLoaded() ) {		    
		    generateTacletPO ();
		}
            }
        } );
        registerAtMenu ( fileMenu, tacletPOItem );


	JMenuItem exit = new JMenuItem("Exit");
	exit.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
	exit.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    exitMain();		    
		}});	


	addSeparator(fileMenu);

	recentFiles = new RecentFileMenu
	    (new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			loadProblem
			    (new File(recentFiles.
				      getAbsolutePath((JMenuItem)e.getSource())));
		    }
		}, MAX_RECENT_FILES, null);

	recentFiles.load(RECENT_FILES_STORAGE);

	registerAtMenu(fileMenu, recentFiles.getMenu());

	addSeparator(fileMenu);
	registerAtMenu(fileMenu, exit);
        return fileMenu;
    }

    protected JMenu createFontSizeMenuEntry() {
        final JMenu fontSize = new JMenu("Font Size");
	
	final JMenuItem smaller = new JMenuItem("Smaller");
	smaller.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Config.DEFAULT.smaller();
		}});
	smaller.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 
	        InputEvent.CTRL_DOWN_MASK));
	
	final JMenuItem larger  = new JMenuItem("Larger");
	larger.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Config.DEFAULT.larger();
		}});
	larger.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 
	        InputEvent.CTRL_DOWN_MASK));
	

	Config.DEFAULT.addConfigChangeListener(
	    new ConfigChangeListener() {
		    public void configChanged(ConfigChangeEvent e) {
			smaller.setEnabled(!Config.DEFAULT.isMinimumSize());
			larger.setEnabled(!Config.DEFAULT.isMaximumSize());
		    }
		});
			
	fontSize.add(smaller);
	fontSize.add(larger);
	return fontSize;
    }
    

    protected JMenu createViewMenu() {
	JMenu view = new JMenu("View");
	view.setMnemonic(KeyEvent.VK_V);

	JMenuItem pretty = new JCheckBoxMenuItem("Use pretty syntax");
	pretty.setToolTipText("If ticked, infix notations are used.");
        pretty.setSelected(PresentationFeatures.ENABLED);
	pretty.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    PresentationFeatures.ENABLED=((JCheckBoxMenuItem)e.getSource()).
			isSelected();
		    makePrettyView();
		}});

	
	
	registerAtMenu(view, pretty);
	addSeparator(view);
		
	registerAtMenu(view, createFontSizeMenuEntry());
	
	final JMenuItem tacletOptionsView = new JMenuItem(TACLET_OPTIONS_MENU_STRING);

	tacletOptionsView.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_M, ActionEvent.CTRL_MASK));
	tacletOptionsView.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    selectMaxTooltipLines();
		    tacletOptionsView.setText(TACLET_OPTIONS_MENU_STRING);
		    //+ "(" +
			// ProofSettings.DEFAULT_SETTINGS.getViewSettings().getMaxTooltipLines()
			// + ")");
		}});
	registerAtMenu(view, tacletOptionsView);

        return view;
    }
    
    protected JMenu createProofMenu() {
        JMenu proof = new JMenu("Proof");
        proof.setMnemonic(KeyEvent.VK_P);

        JMenuItem strategyItem = new JMenuItem("Strategy...");
        strategyItem.setAccelerator(KeyStroke.getKeyStroke
                (KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        strategyItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectStrategy();
            }});
        registerAtMenu(proof, strategyItem);

	JMenuItem close = new JMenuItem("Abandon Task");
	close.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_W, ActionEvent.CTRL_MASK));
	close.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    closeTask();
		}});
	registerAtMenu(proof, close);	

	JMenuItem choiceItem = new JMenuItem("Show active taclet options...");
	choiceItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    showActivatedChoices();
		}});
	registerAtMenu(proof, choiceItem);

	JMenuItem methodContractsItem 
	    = new JMenuItem("Used specifications...");
        methodContractsItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		JFrame fr = new UsedMethodContractsList
		    (mediator.getSelectedProof().getBasicTask(), mediator);
		fr.setVisible(true);
            }});
        registerAtMenu(proof, methodContractsItem);


        return proof;
    }

    protected JMenu createOptionsMenu() {
	JMenu options = new JMenu("Options");
	options.setMnemonic(KeyEvent.VK_O);
	JMenuItem choiceItem = new JMenuItem("Taclet options defaults");
	choiceItem.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_T, ActionEvent.CTRL_MASK));

	choiceItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    selectChoices();
		}});
	registerAtMenu(options, choiceItem);	

	JMenuItem updateSimplifierItem = new JMenuItem("Update Simplifier...");
	updateSimplifierItem.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_U, ActionEvent.CTRL_MASK));

	updateSimplifierItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    configSimultaneousUpdateSimplifier();
		}});
	registerAtMenu(options, updateSimplifierItem);	
    
	ButtonGroup decisionProcGroup = new ButtonGroup();
 
        JMenuItem librariesItem = new JMenuItem("Libraries");
        librariesItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    configLibraries();
                }});
        registerAtMenu(options, librariesItem);  

        simplifyButton.setSelected(ProofSettings.DEFAULT_SETTINGS.
			      getDecisionProcedureSettings().useSimplify());
	simplifyButton.setIcon(IconFactory.simplifyLogo(15));
	decisionProcGroup.add(simplifyButton);
	decisionProcedureOption.add(simplifyButton);

	DecisionProcButtonListener decisionProcButtonListener = 
	    new DecisionProcButtonListener();
	simplifyButton.addActionListener(decisionProcButtonListener);

	icsButton.setIcon(IconFactory.icsLogo(15));
	icsButton.setSelected(ProofSettings.DEFAULT_SETTINGS.
			      getDecisionProcedureSettings().useICS());
	decisionProcGroup.add(icsButton);
	decisionProcedureOption.add(icsButton);
	icsButton.addActionListener(decisionProcButtonListener);
	registerAtMenu(options, decisionProcedureOption);

	JMenuItem computeSpecificationOptions = 
	    ComputeSpecificationView.createOptionMenuItems();
	registerAtMenu(options, computeSpecificationOptions);

	// GENERAL SETTINGS 
	// minimize interaction
	final boolean stupidMode = 
	    ProofSettings.DEFAULT_SETTINGS.getGeneralSettings().stupidMode();
	final JMenuItem stupidModeOption = new JCheckBoxMenuItem("Minimize interaction", 
							   stupidMode);
        mediator.setStupidMode(stupidMode);
            
	stupidModeOption.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                boolean b = ((JCheckBoxMenuItem)e.getSource()).isSelected();
                mediator().setStupidMode(b);
		ProofSettings.DEFAULT_SETTINGS.
		    getGeneralSettings().setStupidMode(b);
	    }});

	registerAtMenu(options, stupidModeOption);
	
	
	// suggestive var names
        // currently removed to avoid confusion
/*        
	final boolean suggestiveVarNames = 
	    ProofSettings.DEFAULT_SETTINGS.getGeneralSettings().suggestiveVarNames();
	final JMenuItem suggVarNamesOption = 
	    new JCheckBoxMenuItem("Suggestive names for aux vars", 
				  suggestiveVarNames);
        VariableNamer.setSuggestiveEnabled(suggestiveVarNames);
            
	suggVarNamesOption.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                boolean b = ((JCheckBoxMenuItem)e.getSource()).isSelected();
                VariableNamer.setSuggestiveEnabled(b);
		ProofSettings.DEFAULT_SETTINGS.
		    getGeneralSettings().setSuggestiveVarNames(b);
	    }});

	registerAtMenu(options, suggVarNamesOption);
*/
	
	// inner/outer renaming
	final boolean outerRenaming = 
	    ProofSettings.DEFAULT_SETTINGS.getGeneralSettings().outerRenaming();
	final JMenuItem outerRenamingOption =
	    new JCheckBoxMenuItem("Outer renaming", outerRenaming);
	Services.setOuterRenaming(outerRenaming);
            
	outerRenamingOption.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                boolean b = ((JCheckBoxMenuItem)e.getSource()).isSelected();
        	Services.setOuterRenaming(b);
		ProofSettings.DEFAULT_SETTINGS.
		    getGeneralSettings().setOuterRenaming(b);
	    }});

	registerAtMenu(options, outerRenamingOption);
			
	addSeparator(options);
    
	// sound settings
	final boolean soundNotification = 
	    ProofSettings.DEFAULT_SETTINGS.getGeneralSettings().soundNotification();
	final JMenuItem soundNotificationOption =
	    new JCheckBoxMenuItem("Sound notification", soundNotification);
	if (notificationManager!=null) {
            notificationManager.setDefaultNotification(soundNotification);
        }
	soundNotificationOption.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
	        boolean b = ((JCheckBoxMenuItem)e.getSource()).isSelected();	       
	        if (notificationManager!=null) {
                    notificationManager.setDefaultNotification(b);                
	        }
	        ProofSettings.DEFAULT_SETTINGS.
	        getGeneralSettings().setSoundNotification(b);	        
        }});
	
	registerAtMenu(options, soundNotificationOption);
	

	// proof assistant
	final JMenuItem assistantOption = new JCheckBoxMenuItem
	    ("Proof Assistant", 
	     ProofSettings.DEFAULT_SETTINGS.
	     getGeneralSettings().proofAssistantMode());

	final ProofAssistantController assistant = new ProofAssistantController
	    (mediator, 
	     ProofSettings.DEFAULT_SETTINGS.getGeneralSettings(),
	     new ProofAssistantAI(), new ProofAssistant());

 	// listen to the state of the assistant in order to hold the
 	// item and state consistent
 	assistant.addChangeListener(new ChangeListener() {
 		public void stateChanged(ChangeEvent e) {
 		    assistantOption.setSelected
 			(((ProofAssistantController)e.getSource()).getState());
 		}
 	    });

	assistantOption.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		ProofSettings.DEFAULT_SETTINGS.getGeneralSettings().
		    setProofAssistantMode
		    (((JCheckBoxMenuItem)e.getSource()).isSelected());
	    }});

	registerAtMenu(options, assistantOption);
	
	addSeparator(options);

	ProofSettings.DEFAULT_SETTINGS.item().addActionListener
	    (new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			ProofSettings.DEFAULT_SETTINGS.saveSettings();
		    }});

	registerAtMenu(options, ProofSettings.DEFAULT_SETTINGS.item());
        return options;
    }


    protected JMenu createHelpMenu() {
	JMenu help = new JMenu("About");
	help.setMnemonic(KeyEvent.VK_A);
	JMenuItem about = new JMenuItem("About KeY");
	about.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    showAbout();
		}});
	help.add(about);	

	JMenuItem license = new JMenuItem("License");
	license.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    showLicense();
		}});
	help.add(license);
        return help;
    }

    protected JMenu createToolsMenu() {
	JMenu tools = new JMenu("Tools");
	tools.setMnemonic(KeyEvent.VK_T);
	getJMenuBar().add(tools);

	JMenuItem extractSpecification = new JMenuItem("Extract Specification");
	extractSpecification.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_E, ActionEvent.CTRL_MASK));

	extractSpecification.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if (mediator().ensureProofLoaded()) {
			//@internal we don't want to block UI just
			//because we are about to calculate a lot of
			//things, now. Also the interactive prover
			//might want to run during the execution of
			//ComputeSpecification
			new Thread(new Runnable() {
				public void run() {
				    ComputeSpecificationView.show(mediator());
				}
			    }).start();
		    }
		}
	    });
	tools.add(extractSpecification);

	JMenuItem specificationBrowser = 
	    new JMenuItem("JML Specification Browser...");
	specificationBrowser.setAccelerator(KeyStroke.getKeyStroke
					    (KeyEvent.VK_J, 
					     ActionEvent.CTRL_MASK));
	specificationBrowser.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    showSpecBrowser();
		}});
	registerAtMenu(tools, specificationBrowser);
        
        JMenuItem nonInterference = new JMenuItem("Check Non-Interference");
	nonInterference.addActionListener(new ActionListener() {
	     public void actionPerformed(ActionEvent e) {
                 BasicTask[] selProofs = proofList.getAllSelectedBasicTasks();
                 if (selProofs.length==2) {
                     new NonInterferenceCheck(selProofs).run();
                 } else {
                    mediator().popupWarning(
                        "Please select 2 proofs", "Non-Interference Check");
                 }
	     }
	});
	tools.add(nonInterference);
        
	return tools;
    }
    
    protected JMenu createDebugMenu() {
    JMenu debug = new JMenu("Debug");
    debug.setMnemonic(KeyEvent.VK_D);
    JMenuItem showSettings = new JMenuItem("Show settings");
    showSettings.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            showSettings();
        }});
    debug.add(showSettings);    
        return debug;
    }
    
    
    /** creates menubar entries and adds them to menu bar */
    private void createMenuBarEntries() {
	JMenuBar menuBar = getJMenuBar();
	menuBar.add(createFileMenu());
	menuBar.add(createViewMenu());
        menuBar.add(createProofMenu());
	menuBar.add(createOptionsMenu());
        menuBar.add(createToolsMenu());
        menuBar.add(Box.createHorizontalGlue());
	menuBar.add(createHelpMenu());
	if (Debug.ENABLE_DEBUG) menuBar.add(createDebugMenu());
    }

    /**
     * returns the toolbar
     */
    public JToolBar getToolBar() {
	return toolBar;
    }

    /** 
     * Sets the content of the current goal view. Do not use this
     * method from outside, take method 
     * {@link #updateGoalView(String, JComponent)} instead
     * (thread safe)
     */
    private void paintGoalView(String borderTitle, JComponent goalViewPane) {
	goalView.setViewportView(goalViewPane);
	goalView.setBorder(new TitledBorder(borderTitle));	
	goalView.validate();	    
	validate();
    }

    /** 
     * updates the view of the sequent being displayed in the main
     * frame 
     */
    private synchronized void updateGoalView(final String borderTitle, 
					     final JComponent goalViewPane) {
	if ( SwingUtilities.isEventDispatchThread () ) {
	    paintGoalView(borderTitle, goalViewPane);
	} else {
	    Runnable sequentUpdater = new Runnable() {
		    public void run() {
 			paintGoalView(borderTitle, goalViewPane);
		    }
		};
	    SwingUtilities.invokeLater(sequentUpdater);
	}
    }
    

    /**
     *  prints the content of the sequent view 
     */
    public void printSequentView(Sequent sequent) {
        SequentPrintFilter filter =
	    new ConstraintSequentPrintFilter ( sequent,
					       mediator ().getUserConstraint ()
					       .getConstraint () );
	final LogicPrinter printer = new LogicPrinter
	    (new ProgramPrinter(null), 
	     mediator().getNotationInfo());
        
        // print shorter attribute names if possible  
        printer.setServices(mediator.getServices());
	
        sequentView.setPrinter(printer, filter, null);
	sequentView.printSequent();	
                      
        updateGoalView("Current Goal", sequentView);
    }
    
    public void applySimplificationOnGoals() {
        if (!mediator.ensureProofLoaded())
            return;
        IteratorOfGoal i = mediator.getProof().openGoals().iterator();
        BuiltInRule simpRule = null;
        while (i.hasNext()) {          
            //AR: %% GUI-Rule separation, many rule instances:
            if (mediator.getProof().getSettings().getDecisionProcedureSettings()
                    .useSimplify())
                simpRule = new SimplifyIntegerRule(false,
                        new JavaDecisionProcedureTranslationFactory());
            else if (mediator.getProof().getSettings().getDecisionProcedureSettings()
                    .useICS())
                simpRule = new ICSIntegerRule(false,
                        new JavaDecisionProcedureTranslationFactory());
            BuiltInRuleApp birApp = new BuiltInRuleApp(simpRule, null, mediator
                    .getUserConstraint().getConstraint());
            mediator.getProof().env().registerRule(simpRule,
                    de.uka.ilkd.key.proof.mgt.AxiomJustification.INSTANCE);						
            i.next().apply(birApp);           
        }      
    }

    public static KeYFileChooser getFileChooser(String title) {
	if (fileChooser==null) {
	    fileChooser = new KeYFileChooser();
	}
	fileChooser.setDialogTitle (title);
	fileChooser.prepare();
	return fileChooser;
    }


    /** saves a proof */
    protected void saveProof() {
	KeYFileChooser jFC = getFileChooser("Choose filename to save proof");
	boolean saved = jFC.showSaveDialog(this);
	if (saved) {
	    saveProof(jFC.getSelectedFile());
	}	
    }

    protected void saveProof(String proofFile) {
        saveProof(new File(proofFile));
    }

    protected void saveProof(File proofFile) {
	String filename = proofFile.getAbsolutePath();    
	ProofSaver saver;
	if (filename.endsWith(".tex"))
	    saver = new ProofSaverLatex(this, filename);
	else saver = new ProofSaver(this, filename);
	String errorMsg = saver.save();
        
	if (errorMsg != null) {
	    notify(new GeneralFailureEvent
                ("Saving Proof failed.\n Error: " + errorMsg));
	}
    }


    protected void loadProblem(File file) {
	recentFiles.addRecentFile(file.getAbsolutePath());
//	Thread t = new Thread(new ProblemLoader(file, this));
//	t.start();
        new ProblemLoader(file, this).run();
    }
    
    
    protected void closeTask() {
	final Proof proof = mediator.getProof();
	if (proof != null) {
	    final TaskTreeNode rootTask = 
		proof.getBasicTask().getRootTask();	
	    proofList.removeTask(rootTask);
	    ((ProofTreeView)proofView.
	     getViewport().getView()).removeProofs(rootTask.allProofs());
	}
    }
    
    protected void generateTacletPO () {
        final KeYFileChooser localFileChooser = getFileChooser ("Choose file to "
								+"load taclets "
								+"from ...");
        boolean loaded = localFileChooser.showOpenDialog ( Main.this );
        if (!loaded)
            return;
        
        final File file = localFileChooser.getSelectedFile ();

        new TacletSoundnessPOLoader(file, this).run();
    }
    

    /**
     * brings window in front and request focus
     */
    private void popup() {
	setState(Frame.NORMAL);
	setVisible(true);
	requestFocus();
    }

    public void addProblem(final de.uka.ilkd.key.proof.ProofAggregate plist) {
        Runnable guiUpdater = new Runnable() {
            public void run() {
                disableCurrentGoalView = true;
                addToProofList(plist);
                setUpNewProof(plist.getFirstProof());
                disableCurrentGoalView = false;
                setProofNodeDisplay();
                popup();
            }
        };
        if (SwingUtilities.isEventDispatchThread())  
            guiUpdater.run();
        else KeYMediator.invokeAndWait(guiUpdater);
    }


    protected Proof setUpNewProof(Proof proof) {
	KeYMediator localMediator = mediator();
	localMediator.setProof(proof);
	return proof;
    }


    private java.util.Hashtable doNotEnable;

    private void setToolBarDisabled() {
        doNotEnable = new java.util.Hashtable(10);
	Component[] cs = toolBar.getComponents ();
	int         i  = cs.length;
	while ( i-- != 0 ) {
            if (!cs[i].isEnabled()) doNotEnable.put(cs[i], cs[i]);
	    cs[i].setEnabled(false);
        }
    }


    private void setToolBarEnabled() {
	Component[] cs = toolBar.getComponents ();
	int         i  = cs.length;
	while ( i-- != 0 ) {
            if (!doNotEnable.containsKey(cs[i]))
	        cs[i].setEnabled(true);
        }
    }


    /**
     * Loads the last opened file
     */
    private final class OpenMostRecentFile extends AbstractAction {
        
        public OpenMostRecentFile() {
            putValue(SMALL_ICON, IconFactory.openMostRecent(toolbarIconSize));            
            putValue(SHORT_DESCRIPTION, "Load last opened file.");            
        }
                        
        public void actionPerformed(ActionEvent e) {
            if (recentFiles != null && 
                    recentFiles.getMostRecent() != null) {
                final String recentFile = 
                    recentFiles.getMostRecent().getAbsolutePath();
                if (recentFile != null) {
                    loadProblem(new File(recentFile));                    
                }
            }
        }
    }

    /**
     * Opens a file dialog allowing to select the file to be loaded
     */
    private final class OpenFile extends AbstractAction {
	public OpenFile() {
            putValue(NAME, "Load ...");            
            putValue(SMALL_ICON, IconFactory.openKeYFile(toolbarIconSize));            
            putValue(SHORT_DESCRIPTION, 
		     "Browse and load problem or proof files.");            
	    putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke
			    (KeyEvent.VK_O, ActionEvent.CTRL_MASK));

        }
                        
        public void actionPerformed(ActionEvent e) {
	    KeYFileChooser keYFileChooser=getFileChooser("Select file "
							 +"to load proof "
							 +"or problem");
	    boolean loaded = keYFileChooser.showOpenDialog(Main.this);
	    if (loaded) {
		File file = keYFileChooser.getSelectedFile(); 
		loadProblem(file);
	    }

        }
    }

    /**
     * Saves the current selected proof.
     */
    private final class SaveFile extends AbstractAction {

	public SaveFile() {
            putValue(NAME, "Save ...");            
            putValue(SMALL_ICON, IconFactory.saveFile(toolbarIconSize));
            putValue(SHORT_DESCRIPTION, "Save current proof.");            
	    putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke
			    (KeyEvent.VK_S, ActionEvent.CTRL_MASK));

	    setEnabled(mediator.getProof() != null);

	    mediator.addKeYSelectionListener(new KeYSelectionListener() {
		    /** focused node has changed */
		    public void selectedNodeChanged(KeYSelectionEvent e) {}

		    /** 
		     * the selected proof has changed. Enable or disable
		     * action depending whether a proof is available or not
		     */ 
		    public void selectedProofChanged(KeYSelectionEvent e) {
			setEnabled(e.getSource().getSelectedProof() != null);
		    }
		});
        }


	public void actionPerformed(ActionEvent e) {
	    if ( mediator().ensureProofLoaded() ) {			
		saveProof();
	    }
	}
    }

    /** The progress monitor that displays a progress bar in
     * a corner of the main window. */
    class MainProgressMonitor implements ProgressMonitor {
	public void setProgress(int progress) {
	    if ( SwingUtilities.isEventDispatchThread () )
		statusLine.setProgress ( progress );
	    else {
		final int v = progress;
		Runnable lineUpdater = new Runnable() {
			public void run() {
			    statusLine.setProgress ( v );
			}
		    };
		SwingUtilities.invokeLater(lineUpdater);
	    }
	}
	
	public void setMaximum(int maximum) {
	    statusLine.setProgressBarMaximum(maximum);
	}
    }

    class MainListener extends WindowAdapter {
	public void windowClosing(WindowEvent e) {
	    exitMain();	    	    
	}
    }    
    
    class MainGUIListener implements GUIListener {
	/** invoked if a frame that wants modal access is opened */

	private void enableMenuBar(JMenuBar  m, boolean b){
	    for (int i = 0; i < m.getMenuCount(); i++){
		if (m.getMenu(i) != null) { // otherwise it is a spacer
		    m.getMenu(i).setEnabled(b);
                }
	    }
	}

	public void modalDialogOpened(GUIEvent e) {
	 
	    if (e.getSource() instanceof ApplyTacletDialog){
		// disable all elements except the sequent window (drag'n'drop !) ...
		enableMenuBar(Main.this.getJMenuBar(), false); 
		Main.this.goalView.setEnabled(false);
		Main.this.proofView.setEnabled(false); 
		Main.this.openGoalsView.setEnabled(false);
		Main.this.userConstraintView.setEnabled(false);
		Main.this.strategySelectionView.setEnabled(false);
		Main.this.ruleView.setEnabled(false);
		setToolBarDisabled();
	    } else {
		// disable the whole main window ...
		Main.this.setEnabled(false);
	    }
	}

	/** invoked if a frame that wants modal access is closed */
	public void modalDialogClosed(GUIEvent e) {
	    if (e.getSource() instanceof ApplyTacletDialog){
		// enable all previously diabled elements ...
		enableMenuBar(Main.this.getJMenuBar(), true); 
		Main.this.goalView.setEnabled(true);
		Main.this.proofView.setEnabled(true); 
		Main.this.openGoalsView.setEnabled(true);
		Main.this.userConstraintView.setEnabled(true);
		Main.this.strategySelectionView.setEnabled(true);
		Main.this.ruleView.setEnabled(true);
		setToolBarEnabled();
	    } else {
		// enable the whole main window ...
	       Main.this.setEnabled(true);
	    }
	}
	
	public void shutDown(GUIEvent e) {
	    Main.this.notify(new ExitKeYEvent());
	    Main.this.setVisible(false);
	}

    }

    /** set to true if the view of the current goal
     * should not be updated */       
    private boolean disableCurrentGoalView = false;


    private static final int MAX_RULE_APPS_LOG10 = 5;


    private synchronized void setProofNodeDisplay() {
	if (!disableCurrentGoalView) {
	    Goal goal;
	    try {
		goal = mediator().getSelectedGoal();
	    } catch(IllegalStateException e) { // there is no proof (yet)
		return;
	    }
	    if ( goal != null &&
		 !mediator.getUserConstraint ().displayClosed ( goal.node () ) ){
		printSequentView(goal.sequent());
	    } else {
		NonGoalInfoView innerNodeView = 
		    new NonGoalInfoView(mediator().getSelectedNode(), 
					mediator());
		updateGoalView("Inner Node", innerNodeView);
	    }
	}
    }

    class MainProofListener implements AutoModeListener, 
                                       KeYSelectionListener,
                                       SettingsListener {
				       
	Logger logger = Logger.getLogger("key.threading");
    
	Proof proof = null;
				       

	/** focused node has changed */
 	public synchronized void selectedNodeChanged(KeYSelectionEvent e) {
            if (mediator().autoMode()) return;
            setProofNodeDisplay();	    
 	}

 	/** the selected proof has changed (e.g. a new proof has been
 	 * loaded) */ 
 	public synchronized void selectedProofChanged(KeYSelectionEvent e) {
	    Debug.out("Main: initialize with new proof");
	    
	    if ( proof != null ) {
	        proof.getSettings().getStrategySettings().removeSettingsListener ( this );
	    }
	    proof = e.getSource().getSelectedProof();
	    if ( proof != null ) {
	        proof.getSettings().getStrategySettings().addSettingsListener( this );
	    }
        
 	    disableCurrentGoalView = false;	    
 	    goalView.setViewportView(null);

	    if ( userConstraint != null )
		userConstraint
		    .removeConstraintTableListener ( constraintListener );

            userConstraint = (proof != null) ? proof.getUserConstraint() :
                null;
            
	    if ( userConstraint != null )
		userConstraint
		    .addConstraintTableListener ( constraintListener );
	    setProofNodeDisplay();
	    makePrettyView();
	    // update label of autoModeButton
	    updateAutoModeConfigButton();
	    updateDecisionProcedureButton();
 	}
 
	/** invoked if automatic execution has started
	 */
	public synchronized void autoModeStarted(ProofEvent e) {
            logger.warn("Automode started");
	    disableCurrentGoalView = true;
	    mediator().removeKeYSelectionListener(proofListener);
	    freezeExceptAutoModeButton ();
	}

	/** invoked if automatic execution has stopped
	 */
	public synchronized void autoModeStopped(ProofEvent e) {
            logger.warn("Automode stopped");
	    if (logger.isDebugEnabled()) {
		logger.debug("From "+Debug.stackTrace());
	    }
	    unfreezeExceptAutoModeButton();
	    disableCurrentGoalView = false;
            setProofNodeDisplay();
	    mediator().addKeYSelectionListener(proofListener);
	}

	/** invoked when a rule has been applied 
	public synchronized void ruleApplied(ProofEvent e) {
	    //	    setProofNodeDisplay();
	}
        */

	/** invoked when the strategy of a proof has been changed */
        public synchronized void settingsChanged ( GUIEvent e ) {
            if ( proof.getSettings().getStrategySettings() == (StrategySettings) e.getSource() ) {
                updateAutoModeConfigButton();
            }
        }

    }
    

    class DecisionProcButtonListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    Proof currentProof = mediator.getProof();
	    ProofSettings currentSettings = ProofSettings.DEFAULT_SETTINGS;
	    if (currentProof!=null) currentSettings = currentProof.getSettings();
        
	    if (e.getSource()==simplifyButton) {
		currentSettings.getDecisionProcedureSettings().
		    setDecisionProcedure(DecisionProcedureSettings.SIMPLIFY);
	    } else if (e.getSource()==icsButton) {
		currentSettings.getDecisionProcedureSettings().
		    setDecisionProcedure(DecisionProcedureSettings.ICS);	
	    }
	    updateDecisionProcedureButton();
	}
    }
    
    public void updateDecisionProcedureButton() {
        Proof currentProof = mediator.getProof();
        DecisionProcedureSettings decSettings = (currentProof==null)?ProofSettings.DEFAULT_SETTINGS.getDecisionProcedureSettings():
            currentProof.getSettings().getDecisionProcedureSettings();
        if (decSettings.useSimplify()) {
            decisionProcedureButton.setIcon
            (IconFactory.simplifyLogo(toolbarIconSize));
            decisionProcedureButton.setToolTipText("Run Simplify");
            decisionProcedureButton.setText("Run Simplify");
        } else {
            decisionProcedureButton.setIcon
            (IconFactory.icsLogo(toolbarIconSize));
            decisionProcedureButton.setToolTipText("Run ICS");
            decisionProcedureButton.setText("Run ICS");
        }
    }
    
    class MainConstraintTableListener implements ConstraintTableListener {
	public void constraintChanged ( ConstraintTableEvent e ) {
	    setProofNodeDisplay ();
	}
    }
    
    class MainTaskListener implements ProverTaskListener {
        public void taskStarted ( String message, int size ) {
            final MainStatusLine sl = getStatusLine ();
            sl.reset ();
            if ( size > 0 ) {
                sl.setProgressPanelVisible ( true );
		getProgressMonitor().setMaximum(size);
            }
            sl.setStatusText ( message );
        }

        public void taskProgress ( int position ) {
            getProgressMonitor().setProgress ( position );
        }

        public void taskFinished () {
            final MainStatusLine sl = getStatusLine ();
            sl.reset ();
        }
    }
	

    public static void evaluateOptions(String[] opt) {
	int index = 0;
	while (opt.length > index) {	    
	    if ((new File(opt[index])).exists()) {
		fileNameOnStartUp=opt[index];
	    } else {
		opt[index] = opt[index].toUpperCase();		
		if (opt[index].equals("NO_DEBUG")) {
		    de.uka.ilkd.key.util.Debug.ENABLE_DEBUG = false;
		} else if (opt[index].equals("DEBUG")) {
		    de.uka.ilkd.key.util.Debug.ENABLE_DEBUG = true;
		} else if (opt[index].equals("NO_ASSERTION")) {
		    de.uka.ilkd.key.util.Debug.ENABLE_ASSERTION = false;
		} else if (opt[index].equals("ASSERTION")) {
		    de.uka.ilkd.key.util.Debug.ENABLE_ASSERTION = true;
		} else if (opt[index].equals("NO_COMPAT")) {
		    compatMode = false;
		} else if (opt[index].equals("COMPAT")) {
		    compatMode = true;
		} else if (opt[index].equals("NO_JMLSPECS")) {
		    jmlSpecs = false;
		} else if (opt[index].equals("AUTO")) {
		    batchMode = true;
		} else {
		    System.out.println("File not found or unrecognized option.\n");
		    System.out.println("Possible parameters are (* = default): ");
		    System.out.println("  no_debug        : disables debug mode (*)");
		    System.out.println("  debug           : enables debug mode");
		    System.out.println("  no_assertion    : disables assertions");
		    System.out.println("  assertion       : enables assertions (*)");
		    System.out.println("  no_compat       : disables compatibility mode for old proofs");
		    System.out.println("  compat          : enables compatibility mode for old proofs");
		    System.out.println("  no_jmlspecs     : disables parsing JML specifications");
		    System.out.println("  auto	          : start proove procedure after initialisation");
		    System.out.println("  <filename>      : loads a .key file");
		    System.exit(-1);
		}		
	    }
	    index++;
	}	
	if (Debug.ENABLE_DEBUG) {
	    System.out.println("Running in debug mode ...");
	} else {
	    System.out.println("Running in normal mode ...");
	}
	if (Debug.ENABLE_ASSERTION) {
	    System.out.println("Using assertions ...");	   
	} else {
	    System.out.println("Not using assertions ...");	   
	}
	if (compatMode)
	{
		System.out.println("Compatibility Mode enabled ...");
	} 

    }
    
    
    /**
     * Glass pane that only delivers events for the status line
     * (i.e. the abort button)
     *
     * This has been partly taken from the GlassPaneDemo of the Java
     * Tutorial
     */
    class BlockingGlassPane extends JComponent {
	GlassPaneListener listener;

	public BlockingGlassPane ( Container contentPane ) {
	    setCursor(new Cursor(Cursor.WAIT_CURSOR));

	    listener = new GlassPaneListener ( this,
					       contentPane );
	    addMouseListener       ( listener );
	    addMouseMotionListener ( listener );
	}
    }


    /**
     * Mouse listener for the glass pane that only delivers events for
     * the status line (i.e. the abort button)
     *
     * This has been partly taken from the GlassPaneDemo of the Java
     * Tutorial
     */
    class GlassPaneListener extends MouseInputAdapter {
	Component currentComponent = null;
	Component glassPane;
	Container contentPane;

	public GlassPaneListener ( Component glassPane,
				   Container contentPane ) {
	    this.glassPane     = glassPane;
	    this.contentPane   = contentPane;
	}

	public void mouseMoved(MouseEvent e) {
	    redispatchMouseEvent(e);
	}

	/*
	 * We must forward at least the mouse drags that started
	 * with mouse presses over the check box.  Otherwise,
	 * when the user presses the check box then drags off,
	 * the check box isn't disarmed -- it keeps its dark
	 * gray background or whatever its L&F uses to indicate
	 * that the button is currently being pressed.
	 */
	public void mouseDragged(MouseEvent e) {
	    redispatchMouseEvent(e);
	}

	public void mouseClicked(MouseEvent e) {
	    redispatchMouseEvent(e);
	}

	public void mouseEntered(MouseEvent e) {
	    redispatchMouseEvent(e);
	}

	public void mouseExited(MouseEvent e) {
	    redispatchMouseEvent(e);
	}

	public void mousePressed(MouseEvent e) {
	    redispatchMouseEvent(e);
	}

	public void mouseReleased(MouseEvent e) {
	    redispatchMouseEvent(e);
	    currentComponent = null;
	}

	private void redispatchMouseEvent(MouseEvent e) {
	    if ( currentComponent != null ) {
		dispatchForCurrentComponent ( e );
	    } else {
		int       eventID        = e.getID();
		Point     glassPanePoint = e.getPoint();

		Point     containerPoint =
		    SwingUtilities.convertPoint(glassPane,
						glassPanePoint, 
						contentPane);
		Component component      =
		    SwingUtilities.getDeepestComponentAt(contentPane,
							 containerPoint.x,
							 containerPoint.y);

		if ( eventID == MouseEvent.MOUSE_PRESSED &&
		     isLiveComponent ( component ) ) {
		    currentComponent = component;
		    dispatchForCurrentComponent ( e );		
		}
	    }
	}

	private boolean isLiveComponent ( Component c ) {
        // this is not the most elegant way to identify the right
        // components, but it scales well ;-)
	    while ( c != null ) {
		if ( (c instanceof JComponent) && 
                     AUTO_MODE_TEXT.equals(((JComponent)c).getToolTipText()) ) 
                    return true;
		c = c.getParent ();
	    }
	    return false;
	}

	private void dispatchForCurrentComponent ( MouseEvent e ) {
	    Point glassPanePoint = e.getPoint();
	    Point componentPoint =
		SwingUtilities.convertPoint( glassPane,
					     glassPanePoint, 
					     currentComponent );
	    currentComponent.dispatchEvent(new MouseEvent(currentComponent,
							  e.getID(),
							  e.getWhen(),
							  e.getModifiers(),
							  componentPoint.x,
							  componentPoint.y,
							  e.getClickCount(),
							  e.isPopupTrigger()));
	}
    }
    
    private final class AutoModeAction extends AbstractAction {

	final Icon startLogo = 
	    IconFactory.autoModeStartLogo ( toolbarIconSize );
	final Icon stopLogo = 
	    IconFactory.autoModeStopLogo ( toolbarIconSize );
    
        public AutoModeAction() {
            putValue("hideActionText", Boolean.TRUE);
            putValue(Action.SHORT_DESCRIPTION, AUTO_MODE_TEXT);
	    putValue(Action.SMALL_ICON, startLogo);
	    
	    
	    setEnabled(mediator.getProof() != null);
	    mediator.addKeYSelectionListener(new KeYSelectionListener() {
		    /** focused node has changed */
		    public void selectedNodeChanged(KeYSelectionEvent e) {}

		    /** 
		     * the selected proof has changed. Enable or disable
		     * action depending whether a proof is available or not
		     */ 
		    public void selectedProofChanged(KeYSelectionEvent e) {
			setEnabled(e.getSource().getSelectedProof() != null);
		    }
		});


	    mediator.addAutoModeListener(new AutoModeListener() {

		    /** 
		     * invoked if automatic execution has started
		     */
		    public void autoModeStarted(ProofEvent e) {
			putValue(Action.SMALL_ICON, stopLogo);
		    }
		    
		    /**
		     * invoked if automatic execution has stopped
		     */
		    public void autoModeStopped(ProofEvent e) {
			putValue(Action.SMALL_ICON, startLogo);
		    }
		    
		});


        }
        
    
        public void actionPerformed(ActionEvent e) {
	    // Unfortunately, when clicking the button twice
	    // (very fast), the glasspane won't be quick
	    // enough to catch the second event. Therefore
	    // we make a second check (which is a %%%HACK)
	    if ( !frozen )
	        mediator().startAutoMode();
            else
                mediator().stopAutoMode();
        }
        
    }
    


    public void loadCommandLineFile() {
	if (fileNameOnStartUp!=null) {
	    loadProblem(new File(fileNameOnStartUp));
	}
    }

    
    public static void main(String[] args) {	
        System.out.println("\nKeY Version " + VERSION);
	System.out.println(COPYRIGHT+"\nKeY is protected by the " +
	      	           "GNU General Public License\n");
 	Main.evaluateOptions(args);
 	Main key = getInstance();
 	key.loadCommandLineFile();
    }

    /**
     * informs the NotificationManager about an event
     * @param event the NotificationEvent 
     */
    public void notify(NotificationEvent event) {
        if (notificationManager != null) {
            notificationManager.notify(event);       
        }
    }
}
