// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import javax.swing.*;

import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ActionEvent;

import java.awt.BorderLayout;
import java.util.*;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.gui.ProofSettings;

public class OCLRefinementSelector extends JDialog {

    private OCLRefinementSettings settings;
    private JCheckBox excThrown;
    private JCheckBox useNull;
    private JCheckBox array;
    private static final OCLMod[] allmods={ExcThrownMod.SINGLETON,
					   NullMod.SINGLETON,
					   ArrayMod.SINGLETON};
    private HashMap modMap =  new HashMap();

    public OCLRefinementSelector() {  
	super(new JFrame(), "Select OCL refinements", true);
	settings = ProofSettings.DEFAULT_SETTINGS.getOCLRefinementSettings();
	layoutOCLRefinementSelector();
	init(settings);
	pack();
	setLocation(70, 70);
	setVisible(true);
    }

    public OCLRefinementSettings getSettings(){
	return settings;
    }

    /** layout */
    protected void layoutOCLRefinementSelector() {

	JButton ok = new JButton("OK");
	ok.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		    settings.store();
		    dispose();
		}
	    });
	JButton cancel = new JButton("Cancel");
	cancel.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		    settings.restore();
		    dispose();
		}
	    });

 	JPanel buttonPanel = new JPanel();
 	buttonPanel.add(ok);
 	buttonPanel.add(cancel);

	excThrown = new JCheckBox("use excThrown for specification of exceptions",
				  settings.excModSet());
	modMap.put(excThrown, ExcThrownMod.SINGLETON);
	useNull = new JCheckBox("use null",settings.nullModSet());
	modMap.put(useNull, NullMod.SINGLETON);
	array = new JCheckBox("use arrays",settings.arrayModSet());
	modMap.put(array, ArrayMod.SINGLETON);

	CheckBoxListener listener = new CheckBoxListener();
	excThrown.addItemListener(listener);
	useNull.addItemListener(listener);
	array.addItemListener(listener);

	JPanel listPanel=new JPanel();
	listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));
	listPanel.add(excThrown);
	listPanel.add(useNull);
	listPanel.add(array);

	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(new JPanel(),BorderLayout.WEST);
	getContentPane().add(listPanel,BorderLayout.CENTER);
	getContentPane().add(buttonPanel,BorderLayout.SOUTH);
    }

    private void init(OCLRefinementSettings oclRefinementSettings) {
	excThrown.setSelected(oclRefinementSettings.excModSet());
	useNull.setSelected(oclRefinementSettings.nullModSet());
	array.setSelected(oclRefinementSettings.arrayModSet());
    }


    class CheckBoxListener implements ItemListener {
	public void itemStateChanged(ItemEvent e) {
	    Object mod=modMap.get(e.getItemSelectable());	    
	    settings.changeFor((OCLMod)mod, e.getStateChange()==ItemEvent.SELECTED);
	}
    }
    
   
}    
