// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uka.ilkd.key.proof.Proof;



public class StrategySelectionView extends JPanel {

    JRadioButton folButton = new JRadioButton("FOL", true);
    JRadioButton javaDLButton = new JRadioButton("Java DL", false);
    JRadioButton oclButton = new JRadioButton("OCL simplification", false);
    JRadioButton specButton = new JRadioButton("Specification extraction", false);
    
    private KeYMediator mediator;
    

    JPanel javaDLOptionsPanel = new JPanel() {
        public void setEnabled(boolean enabled) {
             super.setEnabled(enabled);
             setChildrenEnabled(this, enabled);
        }

        private void setChildrenEnabled(Container container, boolean enabled) {
             for (int i=0; i<container.getComponentCount(); i++) {
                 Component comp = container.getComponent(i);
                 comp.setEnabled(enabled);
                 if (comp instanceof Container)
                     setChildrenEnabled((Container) comp, enabled);
             }

        }
    };
    
    Border loweredetched = 
        BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);
    
    
    public StrategySelectionView () {
	layoutPane();
        refresh(mediator == null ? null : mediator.getSelectedProof());
	setVisible( true );
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent e) {
                numLabel.refresh();
                maxSlider.refresh();
            }
        });
    }

    /** Build everything */    
    private void layoutPane() {
        ButtonGroup loopGroup = new ButtonGroup();
        ButtonGroup methodGroup = new ButtonGroup();
        JLabel label4;
        JLabel label6;
        JRadioButton rdBut9;
        JRadioButton rdBut10;
        JRadioButton rdBut11;
        JRadioButton rdBut12;
        JRadioButton rdBut13;
        JRadioButton rdBut14;

        GridBagLayout gbpanel0 = new GridBagLayout();
        GridBagConstraints gbcpanel0 = new GridBagConstraints();
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        javaDLOptionsPanel.setBorder(BorderFactory.createTitledBorder(""));
        GridBagLayout gbpanel5 = new GridBagLayout();
        GridBagConstraints gbcpanel5 = new GridBagConstraints();
        javaDLOptionsPanel.setLayout(gbpanel5);
        javaDLOptionsPanel.setLayout(new BoxLayout(javaDLOptionsPanel,
                                                   BoxLayout.Y_AXIS));
        
        
        JPanel loopPanel = new JPanel();
        loopPanel.setBorder(BorderFactory.createTitledBorder(
            loweredetched, "Loop treatment"));
        javaDLOptionsPanel.add(loopPanel);
        
        JPanel methodPanel = new JPanel();
        methodPanel.setBorder(BorderFactory.createTitledBorder(
            loweredetched, "Method treatment"));
        javaDLOptionsPanel.add(methodPanel);

        rdBut9 = new JRadioButton("Expand", true);
        loopGroup.add(rdBut9);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 2;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut9, gbcpanel5);
        loopPanel.add(rdBut9);

        rdBut10 = new JRadioButton("Invariant", false);
        loopGroup.add(rdBut10);
        gbcpanel5.gridx = 1;
        gbcpanel5.gridy = 2;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut10, gbcpanel5);
        loopPanel.add(rdBut10);

        rdBut11 = new JRadioButton("None", false);
        loopGroup.add(rdBut11);
        gbcpanel5.gridx = 2;
        gbcpanel5.gridy = 2;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut11, gbcpanel5);
        loopPanel.add(rdBut11);

        rdBut12 = new JRadioButton("Expand", true);
        methodGroup.add(rdBut12);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 6;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut12, gbcpanel5);
        methodPanel.add(rdBut12);

        rdBut13 = new JRadioButton("Contracts", false);
        methodGroup.add(rdBut13);
        gbcpanel5.gridx = 1;
        gbcpanel5.gridy = 6;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut13, gbcpanel5);
        methodPanel.add(rdBut13);

        rdBut14 = new JRadioButton("None", false);
        methodGroup.add(rdBut14);
        gbcpanel5.gridx = 2;
        gbcpanel5.gridy = 6;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut14, gbcpanel5);
        methodPanel.add(rdBut14);
        
        
        
        gbcpanel0.gridx = 4;
        gbcpanel0.gridy = 4;
        gbcpanel0.gridwidth = 3;
        gbcpanel0.gridheight = 7;
        gbcpanel0.fill = GridBagConstraints.BOTH;
        gbcpanel0.weightx = 1;
        gbcpanel0.weighty = 0;
        gbcpanel0.anchor = GridBagConstraints.NORTH;
        gbpanel0.setConstraints(javaDLOptionsPanel, gbcpanel0);

        Box box = Box.createVerticalBox();
        ButtonGroup stratGroup = new ButtonGroup();
        stratGroup.add(folButton);
        stratGroup.add(javaDLButton);
        stratGroup.add(oclButton);
        stratGroup.add(specButton);
        javaDLOptionsPanel.setEnabled(javaDLButton.isSelected());
        javaDLButton.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                javaDLOptionsPanel.setEnabled(javaDLButton.isSelected());
            }
        });
        
        folButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        javaDLButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        javaDLOptionsPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        oclButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        specButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        JTextArea teaser = new JTextArea("This is just a teaser.\n"+
            "It needs to be beautified and hooked up.\nPlease feel free to do so.");
        teaser.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        JButton go = new JButton(Main.autoModeAction);
        go.setAlignmentX(Component.LEFT_ALIGNMENT);
                                      
        box.add(go);
        box.add(numLabel);
        box.add(maxSlider);
        box.add(folButton);
        box.add(javaDLButton);
        box.add(javaDLOptionsPanel);
        box.add(oclButton);
        box.add(specButton);
        box.add(Box.createVerticalGlue());
        box.add(teaser);
        this.add(box);

    }
    

    public void setMediator(KeYMediator mediator) {
        this.mediator = mediator;
        mediator.addKeYSelectionListener(new SelectionChangedListener());
    }


    private final MaxAppLabel numLabel = new MaxAppLabel();
    private final MaxAppSlider maxSlider = new MaxAppSlider();
    
    
    class MaxAppLabel extends JLabel {
        private final static String TEXT = "Max. rule applications: ";
    
        public MaxAppLabel() {
            super(TEXT);
            setFont(getFont().deriveFont(9F));
            setAlignmentX(Component.LEFT_ALIGNMENT);
        }
    
        public void setLimit(int max) {
            setText(TEXT+max);
        }
        
        public void refresh() {
            setLimit(mediator.getMaxAutomaticSteps());
        }
    }
    
    class MaxAppSlider extends JSlider {
        private static final int MAX_RULE_APPS_LOG10 = 5;
        
        public MaxAppSlider() {
            super(JSlider.HORIZONTAL, 0, MAX_RULE_APPS_LOG10*9, 0);

            // set up slider labels
            Hashtable labelTable = new Hashtable();

            for ( int n = 0; n <= MAX_RULE_APPS_LOG10; n++ ) {
                int val = (int)Math.pow(10, n);
                JLabel l = new JLabel ( ""+val );
                l.setFont(l.getFont().deriveFont(9F));
                labelTable.put(new Integer ( n*9 ), l);
            }

            setLabelTable ( labelTable );
            setPaintLabels ( true );

            // show ticks
            setMajorTickSpacing ( 9 );
            setMinorTickSpacing ( 1 );
            setPaintTicks ( true );

            // add change listener
            addChangeListener( new ChangeListener() {
                public void stateChanged ( ChangeEvent e ) {
                    int n = getValue();
                    int major = n / 9;
                    int minor = n % 9;
                    int val = (minor+1)*(int)Math.pow(10, major);
                    numLabel.setLimit( val );
                    mediator.setMaxAutomaticSteps ( val );
                }
            });

            setAlignmentX(Component.LEFT_ALIGNMENT);
        }
        
        public void setPos(int maxRuleApps) {
            if ( maxRuleApps > 0 ) {
                int major = (int)(Math.log ( maxRuleApps ) / Math.log ( 10 ));
                int minor = maxRuleApps / (int)Math.pow(10, major) - 1;
                int initPos = Math.max ( 0, Math.min ( MAX_RULE_APPS_LOG10*9,
                                                       major * 9 + minor ));
                setValue(initPos);
            }
        }
        
        public void refresh() {
            setPos(mediator.getMaxAutomaticSteps());
        }
    
    }

    /**
     * TODO: here should be a settings listener listening to strategy setting changes
     * Therefore we have to wait until ProofSettings have been refactored (see KeY-Wiki) 
     * 
     *(Currently changes made in the old slider are not reported to this view)
     */
    private final class SelectionChangedListener 
        implements KeYSelectionListener {        
        
        public void selectedNodeChanged(KeYSelectionEvent e) {}

        public void selectedProofChanged(KeYSelectionEvent e) {
            refresh(e.getSource().getSelectedProof());            
        }        
    }

    
    /** performs a refresh of all elements in this tab */
    public void refresh(Proof proof) {
        if (proof == null) {            
            enableAll(false);                
        } else {
            // TODO: adapt FOL, JAVADL Options                        
            maxSlider.refresh();
            numLabel.refresh();
            enableAll(true);           
        }
    }

    /**
     * enables or disables all components
     * @param enable boolean saying whether to activate or
     * deactivate the components
     */
    private void enableAll(boolean enable) {     
        maxSlider.setEnabled(enable);
        folButton.setEnabled(enable);
        javaDLButton.setEnabled(enable);
        javaDLOptionsPanel.setEnabled(enable);
        oclButton.setEnabled(enable);
        specButton.setEnabled(enable);
    }
    
}
