// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/* Generated by Together */

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.java.JavaProgramElement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.PrettyPrinter;
import de.uka.ilkd.key.java.NameAbstractionTable;
import java.io.StringWriter;
import java.io.IOException;

public class JavaBlock {
    
    public static final JavaBlock EMPTY_JAVABLOCK=new JavaBlock();
    private final JavaProgramElement prg;


    /** create a new empty JavaBlock as singleton 
     */
    private JavaBlock() {
	this.prg=null;
    }

    /** create a new JavaBlock 
     * @param prg the root JavaProgramElement for this JavaBlock
     */
    private JavaBlock(JavaProgramElement prg) {
	this.prg=prg;
    }

    /** create a new JavaBlock 
     * @param prg the root StatementBlock for this JavaBlock.
     * TacletIndex relies on <code>prg</code> being indeed a StatementBlock.
     */
    public static JavaBlock createJavaBlock(StatementBlock prg) {
	if (prg==null) {
	    return EMPTY_JAVABLOCK;	   
	} 
	return new JavaBlock(prg);

    }
    

    public boolean isEmpty() {
	if ((program() instanceof StatementBlock))  {
	    return ((StatementBlock)program()).isEmpty();
	}
	return this==EMPTY_JAVABLOCK;
    }
    
    public int size() {
	if ((program() instanceof StatementBlock))  {
	    return ((StatementBlock)program()).getChildCount();
	}
	return 0;
    }
    
    
    public Namespace namespace() {
	return null;
    }
    
    /** converts a parser generated namespace into a KeY namespace */
    public Namespace convertNamespace() {
	return null;
    }

    /** returns the hashCode */
    public int hashCode() {
    	int result = 17;
    	result = 37 * result + ((program()==null) ? 0 : program().hashCode());
    	return result;
    }

    /** returns true iff the program elements are equal */
    public boolean equals(Object o) {
        if ( o == this )
            return true;
		if (!(o instanceof JavaBlock) || o == null) {
		    return false;
		}
		JavaBlock cmp = (JavaBlock)o;
		return cmp.program().equals(program());
    }

    /** returns true if the given ProgramElement is equal to the
     * one of the JavaBlock modulo renaming (see comment in SourceElement)
     */ 
    public boolean equalsModRenaming(Object o, 
				     NameAbstractionTable nat) {
        if (!(o instanceof JavaBlock) || o == null) {
            return false;
        }
        final JavaBlock cmp = (JavaBlock)o;
        return equalsModRenaming(cmp.program(), nat);
    }


    /** returns true if the given ProgramElement is equal to the
     * one of the JavaBlock modulo renaming (see comment in SourceElement)
     */ 
    private boolean equalsModRenaming(JavaProgramElement pe,
				     NameAbstractionTable nat) {
	if (pe == null && program() == null) {
	    return true;
	} else if ((pe == null && program()!=null) ||
	    (program() == null && pe != null)) {	    
	    return false;
	}
	return program().equalsModRenaming(pe, nat);
    }

    /** returns the java program 
     * @return the stored JavaProgramElement
     */
    public JavaProgramElement program() {
	return prg;
    }

    /** toString */
    public String toString() {
	if (this==EMPTY_JAVABLOCK) return "";
	StringWriter sw=new StringWriter();
	try {
	    PrettyPrinter pp=new PrettyPrinter(sw, true);
	    pp.setIndentationLevel(0);
	    prg.prettyPrint(pp);
	} catch (IOException e) {
	    System.err.println("toString of JavaBlock failed due to :"+e);
	    e.printStackTrace();
	}
	return sw.toString();
    }

}
