// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic;

import junit.framework.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.op.*;

/** class tests the to-array visitor
*/


public class TestToArrayVisitor extends TestCase {
    private final TermFactory tf=TermFactory.DEFAULT;
    
    private Term term;
    private Object[] expected;

    public TestToArrayVisitor(String name) {
	super(name);
    }

    public void setUp() {
	// all[x](P(f(x),g(y,h)) and not(Q(z)))
	Sort nat = new PrimitiveSort(new Name("Nat"));
	Sort bool = Sort.FORMULA;
	
	Function func_f=new RigidFunction(new Name("f"),nat,new Sort[]{nat});
	Function func_g=new RigidFunction(new Name("g"),nat,new Sort[]{nat,nat});
	Function func_h=new RigidFunction(new Name("h"),nat,new Sort[]{});
	Function func_P=new RigidFunction(new Name("P"),bool,new Sort[]{nat,nat});
	Function func_Q=new RigidFunction(new Name("Q"),bool,new Sort[]{nat});
	LogicVariable var_x=new LogicVariable(new Name("x"),nat);
	LogicVariable var_y=new LogicVariable(new Name("y"),nat);
	LogicVariable var_z=new LogicVariable(new Name("z"),nat);
	
	Term term_x=tf.createFunctionTerm(var_x,new Term[]{});
	Term term_y=tf.createFunctionTerm(var_y,new Term[]{});
	Term term_z=tf.createFunctionTerm(var_z,new Term[]{});	
	Term term_f=tf.createFunctionTerm(func_f,new Term[]{term_x});
	Term term_h=tf.createFunctionTerm(func_h,new Term[]{});
	Term term_g=tf.createFunctionTerm(func_g,new Term[]{term_y,term_h});
	Term term_P=tf.createFunctionTerm(func_P,new Term[]{term_f,term_g});
	Term term_Q=tf.createFunctionTerm(func_Q,new Term[]{term_z});
	
	Term term_not=tf.createJunctorTerm(Op.NOT,new Term[]{term_Q});
	Term term_and=tf.createJunctorTerm(Op.AND,new Term[]{term_P,term_not});	
	
	term=tf.createQuantifierTerm(Op.ALL,new LogicVariable[]{var_x},term_and);

	expected=new Object[] {
	    term_x,
	    term_f,
	    term_y,
	    term_h,
	    term_g,
	    term_P,
	    term_z,
	    term_Q,
	    term_not,
	    term_and,
	    term
	};
	
}


    public void testToArrayVisitor() {
	ToArrayVisitor tav=new ToArrayVisitor();
	term.execPostOrder(tav);
	Object[] result=tav.getAllSubTerms();
	if (result.length!=expected.length) fail();
	for (int i=0; i<result.length; i++)
	    assertEquals(expected[i],result[i]);
    }

   

}
