// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.*;
/**
 * visitor for <t> execPostOrder </t> of 
 * {@link de.uka.ilkd.key.logic.Term}. Called with that method
 * on a term, the visitor delivers all pairs of variables 
 * (x,y) in that
 * term where the following condition holds: y is bound and x is in the
 * scope of that binding; additionally (y,y) is delivered if y is bound
 * and no variable occurs in the scope of y
 */
public class VarInScopeCollector extends Visitor{
  
    private SetOfInScopePair bdVars =
	SetAsListOfInScopePair.EMPTY_SET; 

    private IteratorOfLogicVariableRepresentative varsIn(Term t) {
	LVRCollector vcv = new LVRCollector();
	t.execPostOrder(vcv);
	return vcv.varIterator();	
    }

    private void addVarsBoundHere(Term visited, int subTerm) {
	for (int j=0; j<visited.varsBoundHere(subTerm).size(); j++) {
	    boolean added = false;
	    IteratorOfLogicVariableRepresentative varIt 
		= varsIn(visited.sub(subTerm));
	    while (varIt.hasNext()) {
		QuantifiableVariable bound 
		    = visited.varsBoundHere(subTerm).getQuantifiableVariable(j);
		LogicVariableRepresentative inScope = varIt.next();
		added = true;
		bdVars = bdVars.add(new InScopePair(inScope, bound)); 
	    }
	    if (added) {
		bdVars = bdVars.add(new InScopePair
		    (visited.varsBoundHere(subTerm).
                     getQuantifiableVariable(j),
		     visited.varsBoundHere(subTerm)
		     .getQuantifiableVariable(j))); 
	    }
	}
    }

    /**
     * only called by execPostOrder in Term.
     */
    public void visit(Term visited) {
	for (int i=0; i<visited.arity(); i++) {
	    addVarsBoundHere(visited, i);
	}
    }
    
    /**
     * After the visitor has been called by the execPostOrder method of a
     * term, all pairs of variables (x,y) with the following condition: y is
     * bound and x is in the scope of that binding; additionally (y,y) is
     * delivered if y is bound.
     */
    public SetOfInScopePair getVarInScope(){
	return bdVars;
    }

}
