// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;


/**
 * This implements a general conditional operator <tt>if (phi) (t1) (t2)</tt>
 */
public class IfThenElse extends Op {
    
    /**
     * creates the default if-else operator
     */
    IfThenElse () {
        super ( new Name ( "if-then-else" ) );
    }

    /**
     * creates an if-else operator of the given name
     */
    IfThenElse (Name name) {
        super ( name );
    }

    public boolean validTopLevel (Term term) {
        final Sort s0 = term.sub ( 0 ).sort ();
        final Sort s1 = term.sub ( 1 ).sort ();
        final Sort s2 = term.sub ( 2 ).sort ();
        
        // TODO: like in <code>ConjCond</code>, but this is really bad!!! /PR
        return term.arity () == arity ()
               && s0 == Sort.FORMULA
               && ( s1 == Sort.FORMULA ) == ( s2 == Sort.FORMULA );
        
//
//        if ( term.arity () != arity ()
//             || s0 != Sort.FORMULA
//             || !( s1.extendsTrans ( s2 ) || s2.extendsTrans ( s1 )
//                   || term.sub ( 1 ).sort () instanceof ProgramSVSort
//                   || term.sub ( 2 ).sort () instanceof ProgramSVSort
//                   || term.sub ( 1 ).sort () == AbstractMetaOperator.METASORT
//                   || term.sub ( 2 ).sort () == AbstractMetaOperator.METASORT ) )
//            return false;
//        return true;
    }

    public Sort sort (Term[] term) {
        final Sort s2 = term[1].sort ();
        final Sort s3 = term[2].sort ();
        if ( s2.extendsTrans ( s3 ) || s2 instanceof ProgramSVSort
             || s2 == AbstractMetaOperator.METASORT )
            { return s3; }
        if ( s3.extendsTrans ( s2 ) || s3 instanceof ProgramSVSort
             || s3 == AbstractMetaOperator.METASORT ) {
            return s2;
        } else {
            // TODO: this is mostly nonsense /PR
            return s3;
            
//            throw new IllegalArgumentException ( "Cannot determine sort of invalid term (sort of the "
//                                                 + "'then' and 'else' part are not compatible\n "
//                                                 + "("
//                                                 + term[1].sort ()
//                                                 + " and "
//                                                 + term[2].sort ()
//                                                 + ")" );
        }
    }

    public int arity () {
        return 3;
    }   
}
