// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic.sort;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.op.*;

public abstract class AbstractNonCollectionSort extends AbstractSort implements
        NonCollectionSort {

    protected CollectionSort setSort;

    protected CollectionSort bagSort;

    protected CollectionSort sequenceSort;
   
    private boolean symbolsCreated = false;

    /** creates a Sort (with a new equality symbol for this sort) */
    public AbstractNonCollectionSort(Name name) {
        super(name);
        setSort = AbstractCollectionSort.createSetSort(this);
        bagSort = AbstractCollectionSort.createBagSort(this);
        sequenceSort = AbstractCollectionSort.createSequenceSort(this);
    }

    public CollectionSort getSetSort() {
        return setSort;
    }

    public CollectionSort getBagSort() {
        return bagSort;
    }

    public CollectionSort getSequenceSort() {
        return sequenceSort;
    }

    /**
     * Create the symbols defined by this sort, insert them into the namespace
     * "p_func_ns"
     * 
     * @param p_func_ns
     *            Namespace to which functions and predicates should be added to
     */
    public void createSymbols(Namespace p_func_ns, Namespace sort_ns) {        
        super.createSymbols(p_func_ns, sort_ns);
        if (!symbolsCreated) {
            ListOfSortDependingSymbol l0 = SLListOfSortDependingSymbol.EMPTY_LIST;
            ListOfSortDependingSymbol l1;

            l1 = getSetSort().createSymbols();
            if (l1 == null)
                return;
            l0 = l0.prepend(l1);

            l1 = getBagSort().createSymbols();
            if (l1 == null)
                return;
            l0 = l0.prepend(l1);

            l1 = getSequenceSort().createSymbols();
            if (l1 == null)
                return;
            l0 = l0.prepend(l1);            

            addSymbols(l0, p_func_ns);

            symbolsCreated = true;
        }
    }  
}
