// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import tudresden.ocl.OclTree;
import tudresden.ocl.check.types.Basic;
import tudresden.ocl.check.types.Type;
import tudresden.ocl.parser.node.Node;
import de.uka.ilkd.key.ProverFacade;
import de.uka.ilkd.key.casetool.HashMapOfClassifier;
import de.uka.ilkd.key.casetool.JavaOCLTypeMap;
import de.uka.ilkd.key.casetool.UMLOCLBehaviouralFeature;
import de.uka.ilkd.key.casetool.UMLOCLClassifier;
import de.uka.ilkd.key.java.TypeConverter;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.logic.JavaBlock;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.ArrayOp;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.LogicVariable;
import de.uka.ilkd.key.logic.op.Modality;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.sort.ArraySort;
import de.uka.ilkd.key.logic.sort.ArraySortImpl;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.ocl.ocldltranslator.KeyOclConstraintTranslator;
import de.uka.ilkd.key.ocl.ocldltranslator.KeyOclExpressionTranslator;
import de.uka.ilkd.key.ocl.ocldltranslator.KeyTranslationResult;
import de.uka.ilkd.key.ocl.ocldltranslator.TranslationResult;


public class ArrayMod extends OCLMod{

    public static final ArrayMod SINGLETON=new ArrayMod();

    public static final String ARRAYOF_PREFIX="KeYArrayOf";
    private String constraint;

    public JavaBlock getMethodCallJavaBlock(KeyOclConstraintTranslator 
					    consTransl, String methodReturnType,
					    ProgramVariable[] argProgVars, 
					    String constrainedOperation, 
					    String constrainedType) {
	return null;
    }


    public TranslationResult getTranslationFor(DetailedConstrTransl
					       consTransl,
					       OclTree oclTree, 
					       boolean useDiamondForm, 
					       boolean ignorePost,
					       boolean 
					       oclTreeIsSimpleExpression,
					       Modality modality){
	return null;
    }


    public boolean translateOCL2DL(KeyOclExpressionTranslator exprTransl, 
				Node n, Node appliedTo,
				String featureName,
				Vector parameterNodes, boolean atpre) {

	TermFactory tf = exprTransl.termFactory();
	KeyTranslationResult objectTranslation = (KeyTranslationResult) 
	    exprTransl.getTranslationForNode(appliedTo);
	Term objectTerm = objectTranslation.getTerm();
	Sort objectSort = objectTerm.sort();	
        ProverFacade proverFacade = exprTransl.proverFacade();
	if (objectSort instanceof ArraySort) {
	    Term resultTerm = null;
            KeyTranslationResult resultTranslation;
            resultTranslation = new KeyTranslationResult();                                     
	    if ("get".equals(featureName)) {
                Node param = (Node) parameterNodes.get(0);
                    Term paramTerm = ((KeyTranslationResult) 
                            exprTransl.getTranslationForNode(param)).getTerm();                    
	        if (atpre) {
                    ArrayOp aop = ArrayOp.getArrayOp(objectTerm.sort());
                    Sort[] argSorts = new Sort[]{aop.arraySort(), proverFacade.getSortForOCLInteger()};
	            Function getAtPre = proverFacade.getRigidFunctionForFeature("get_pre",
                             argSorts,
                             ((ArraySort)objectSort).elementSort());
                    exprTransl.occuringMethodAtPreFunctions.put(getAtPre, aop);
                    resultTerm = tf.createFunctionTerm(getAtPre, objectTerm, paramTerm);
/*                    Term var1 = tf.createVariableTerm(new LogicVariable(new Name("arr0"), objectSort));
                    Term var2 = tf.createVariableTerm(new LogicVariable(new Name("int0"), proverFacade.getSortForOCLInteger()));
                    Term axiomTerm = tf.createFunctionTerm(getAtPre, var1, var2);
                    axiomTerm = tf.createEqualityTerm(axiomTerm, tf.createArrayTerm
                                   (ArrayOp.getArrayOp(objectTerm.sort()), var1, var2));
                    axiomTerm = exprTransl.constraintTranslator().createAllClosureOfTerm(axiomTerm);
                    resultTranslation.addAxiom(axiomTerm);*/
	        } else {
	            resultTerm = tf.createArrayTerm
	            (ArrayOp.getArrayOp(objectTerm.sort()), objectTerm,
	                    paramTerm);
	        }
	    }
 	    if ("length".equals(featureName)) {
		TypeConverter tc = exprTransl.proverFacade()
		    .services().getTypeConverter();	       
		ProgramVariable at = exprTransl.proverFacade()
		    .getAttribute("length",
				  tc.getKeYJavaType
				  (objectTerm));
 		resultTerm = tf.createAttributeTerm(at, objectTerm);
 	    }
	    resultTranslation.setTerm(resultTerm);
	    exprTransl.setTranslationForNode(n, resultTranslation);	
	    return true;
	}
	return false;
    }

    public boolean transformOCL(String constraint) {
	this.constraint=constraint;
	return false;
    }

    public String getOCLConstraint() {
	return constraint;
    }

    private UMLOCLClassifier getArrayClassifier(Type clf) {
	String arrayClassName = ARRAYOF_PREFIX+clf.toString();
	UMLOCLClassifier arrayClass = new UMLOCLClassifier(arrayClassName,
							   arrayClassName);
	Type intType = tudresden.ocl.check.types.Basic.INTEGER;	
	UMLOCLBehaviouralFeature f = new UMLOCLBehaviouralFeature("get", clf,
								  intType);
	
	arrayClass.addFeature(f.getName()+"(int)",f); 
	
	f = new UMLOCLBehaviouralFeature("length", intType);
	
	arrayClass.addFeature(f.getName()+"()",f); 
	return arrayClass;
    }


    public void modifyAllTypes(HashMapOfClassifier classifiers) {
    	Iterator it = classifiers.values().iterator();
	LinkedList newClassifiers = new LinkedList();
	while (it.hasNext()) {
	    UMLOCLClassifier clf = (UMLOCLClassifier)it.next();	    	    	    
	    newClassifiers.add(getArrayClassifier(clf));
	}
	for (int i=0; i<JavaOCLTypeMap.getBasicTypes().length; i++) {
	    Type t = JavaOCLTypeMap.getBasicTypes()[i];
	    newClassifiers.add(getArrayClassifier(t));
	}
	newClassifiers.add(getArrayClassifier(Basic.STRING));
	Iterator newIt = newClassifiers.iterator();
	while (newIt.hasNext()) {
	    UMLOCLClassifier clf = (UMLOCLClassifier) newIt.next();
	    classifiers.putClassifier(clf.getName(), clf);
	}
	
    }


    public KeYJavaType getKeYJavaType(String s,
				      KeyOclExpressionTranslator exprTransl) {
	boolean isNowArray=false;
        while (s.startsWith(ARRAYOF_PREFIX)) {
	    s = s.substring(ARRAYOF_PREFIX.length());
	    if ("Integer".equals(s)) s="int";
	    if ("Boolean".equals(s)) s="boolean";
	    if ("String".equals(s)) s="java.lang.String";
	    s = s+"[]";	   
	    isNowArray=true;
	}
	if (isNowArray) {
	    KeYJavaType kjt = exprTransl.getKeYJavaTypeForNonCollectionType(s);
	    return kjt;
	} else {
	    return null;
	}
    }

    public Type getTypeFor(recoder.abstraction.Type type, 
			   HashMapOfClassifier classes,
			   JavaOCLTypeMap caller){
    	if (type instanceof recoder.abstraction.ArrayType) {
	    recoder.abstraction.Type recBaseType
		= ((recoder.abstraction.ArrayType)type).getBaseType();
	    Type t = caller.getTypeFor(recBaseType, classes);
	    if (t==null) {
		if (recBaseType instanceof recoder.abstraction.ArrayType) {
		    t = getTypeFor(recBaseType, classes, caller);
		} else {
		    t = new UMLOCLClassifier(recBaseType.getName(), recBaseType.getFullName());
		}
		classes.putClassifier(recBaseType.getName(), (UMLOCLClassifier) t);
		UMLOCLClassifier clf = getArrayClassifier(t);
		classes.putClassifier(clf.getName(), clf);	    
	    }
	    String tString=t.toString();

	    Type res = classes.getClassifier(ArrayMod.ARRAYOF_PREFIX
					     +tString);
	    if (res == null) {
		res = getArrayClassifier(t);
		classes.putClassifier(((UMLOCLClassifier)res).getName(), 
				      (UMLOCLClassifier)res);
	    }
	    return res;
	}
	return null;
    }


}
