// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;
import de.uka.ilkd.key.ProverFacade;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.Modality;
import de.uka.ilkd.key.logic.op.ProgramVariable;

import java.util.*;
import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;
import tudresden.ocl.codegen.*;
import tudresden.ocl.*;
import tudresden.ocl.normalize.*;
import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

public class DetailedConstrTransl extends KeyOclConstraintTranslator{

    private Vector listeners=new Vector();

    public DetailedConstrTransl(ProverFacade facade,			      
				ReprModelMethod method){
	super(facade, method);
    }

    public void addListener(OCLMod listener) {
	listeners.add(listener);
    }

    public void addListeners(OCLMod[] l) {
	for (int i=0; i<l.length; i++) {
	    listeners.add(l[i]);
	}
    }

    public JavaBlock getMethodCallJavaBlock(ProgramVariable[] argProgVars) {
 	Iterator it=listeners.iterator();
	methodParameters = collectMethodParameters
	    (contextMethod.getContainingClass().getClassName());
 	while (it.hasNext()) {
	    OCLMod currentMod=(OCLMod)it.next();
	    JavaBlock jb 
		= currentMod.getMethodCallJavaBlock(this, methodReturnType, 
						    methodParameters, 
						    constrainedOperation,
						    constrainedType);
	    if (jb!=null) {
		return jb;
	    }
	}
	return super.getMethodCallJavaBlock(argProgVars);
    }

    public TranslationResult getSuperTranslationFor(OclTree oclTree, 
				  boolean useDiamondForm, 
				  boolean ignorePost,
				  boolean 
				  oclTreeIsSimpleExpression,
                                  Modality modality){
	return super.getTranslationFor(oclTree, useDiamondForm, 
				       ignorePost, 
				       oclTreeIsSimpleExpression,
                                       modality);
    }

    public TranslationResult getTranslationFor(OclTree oclTree, 
					       boolean useDiamondForm, 
					       boolean ignorePost,
					       boolean 
					       oclTreeIsSimpleExpression,
                                               Modality modality){
 	Iterator it=listeners.iterator();
 	while (it.hasNext()) {
	    OCLMod currentMod=(OCLMod)it.next();
 	    TranslationResult tr
		=currentMod.getTranslationFor(this, oclTree, 
					      useDiamondForm, 
					      ignorePost, 
					      oclTreeIsSimpleExpression,
					      modality);
	    if (tr!=null) {
		return tr;
	    }
 	}	
	return super.getTranslationFor(oclTree, useDiamondForm, 
				       ignorePost, 
				       oclTreeIsSimpleExpression, modality);	
    }
    
    /**
     * A template method that uses an expression translator that
     * considers operations like excThrown or attributes like null differently
     *
     * @param predefinedVarNames the variables that are defined
     * @param predefinedVarTypes the types of variables that are defined
     * @param tControl the control object to support modified translations
     * @return the translator object for expressions to be used
     */    
    protected KeyOclExpressionTranslator 
	getKeyOclExpressionTranslator(Vector predefinedVarNames,
				      Vector predefinedVarTypes,
				      DefaultTranslationControl tControl){

	DetailedExprTransl det=new DetailedExprTransl(tree, this,
						      predefinedVarNames,
						      predefinedVarTypes,
						      tControl);
	Iterator it=listeners.iterator();
	while (it.hasNext()) {
	    det.addListener((OCLMod)it.next());
	}
	return det;
    }


}


