// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.cspec.ComputeSpecification;
import de.uka.ilkd.key.cspec.ExcerptMethodCallMod;
import de.uka.ilkd.key.gui.ProofSettings;
import de.uka.ilkd.key.logic.JavaBlock;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.OCLMod;
import de.uka.ilkd.key.util.Debug;

/**
 * Represents a proof obligation originating from a request of
 * computing the specification of a method as input for the prover.
 * @author Andr&eacute; Platzer
 * @version 1.0, 2003-02-24
 * @see de.uka.ilkd.key.cspec.ComputeSpecification
 */
public class ComputeSpecificationPO extends OCLPostPO {

    // HACK duplicate data storage here and in super class. But still keep private
    private ReprModelMethod methRepr;
    private String precondition;
    private String postcondition;
    private String xmifile;

    /**
     * creates a new representation of a proof obligation from a
     * request of computing the specification.
     * @param name the name used to identify the proof obligation
     * @param aMethRepr the representation of a model method
     * originating this proof obligation. That is the method to
     * compute a specification for.
     * @param precondition the precondition already specified for the
     * method (or "true" if none).
     * @param postcondition the precondition already specified for the
     * method (or "true" if none).
     * @param xmiFileName The name of an XMI file name that can be
     * used to read the underlying model.
     */
    public ComputeSpecificationPO(String name, ReprModelMethod aMethRepr,
		     String precondition, 
		     String postcondition, 
		     String xmiFileName) {
	super(name, aMethRepr, precondition, postcondition, Op.DIA, 
              xmiFileName);
	this.methRepr=aMethRepr;
	this.precondition=precondition;
	this.postcondition=postcondition;
	this.xmifile=xmiFileName;
    }

    /**
     * reads the proof obligation, i.e., invokes the OCL-DL-translation on
     * the OCL constraint of this OCL proof obligation.
     */
    public void read(ModStrategy mod) {
	final InitConfig initConf = getInitConfig();
	if (initConf==null) {
	    throw new IllegalStateException("KeYFile: InitConfig not set.");
	}
	Debug.out("1) program variables are", initConf.progVarNS().getProtocolled());
	Debug.out("1) variables are", initConf.varNS());
	
	final ExcerptMethodCallMod excerpt = new ExcerptMethodCallMod();
	{
	    //@internal the following lines will read the Java sources, thus extract programVariables and inform excerpt of the method call. @xxx is there a better way?
	    OCLMod mods[] = ProofSettings.DEFAULT_SETTINGS
		.getOCLRefinementSettings().getOCLMods();
	    final List l = new ArrayList(mods.length + 1);
	    l.add(excerpt);
	    l.addAll(Arrays.asList(mods));
	    mods = (OCLMod[]) l.toArray(new OCLMod[0]);
	    final Object forgetAbout = generateMethodPO(initConf.createFacadeForTransl(),
							mods,
							methRepr, 
							"prepost",
							precondition, 
							postcondition, 
							Op.DIA, xmifile);
	}

	Debug.out("2) program variables are", initConf.progVarNS().getProtocolled());
	Debug.out("2) variables are", initConf.varNS());

	final JavaBlock methodCallJavaBlock = excerpt.getMethodCall();
	final Term resultTerm = new ComputeSpecification()
	    .createSpecificationComputationTerm(methodCallJavaBlock, 
						initConf.progVarNS());
	//@todo resultTerm = precondition -> resultTerm (translating away OCL)
	setProofObligation(resultTerm);
    }

}
