// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import java.util.Iterator;

import de.uka.ilkd.key.proof.mgt.Contract;
import de.uka.ilkd.key.proof.mgt.ContractSet;
import de.uka.ilkd.key.rule.SetOfTaclet;
import de.uka.ilkd.key.rule.Taclet;


/** represents an entity read to produce an environment to read a proof
 * obligation. Environment means the initial configuration of a prover 
 * containing namespaces and Java model.
 */
public class MethSpecInput implements EnvInput{
     
    /** The initial configuration for a prover to use for reading and modify*/
    protected InitConfig initConfig = null;
    
    private String name;   

    /** Creates a new representation of a Method Specification 
     * input with a name describing the kind of input.
     */
    public MethSpecInput(String name, InitConfig initConfig) {
	this.name=name;
	this.initConfig = initConfig;
    }

    /** returns the initial configuration that has been the result of 
     * reading the input.
     */
    public InitConfig getInitConfig() {
	return initConfig;
    }

    /** sets the initial configuration the read environment input should be
     * added to.
     */
    public void addTo(InitConfig i) {
	this.initConfig = i;
    }

    /** reads the input using the given modification strategy, i.e.,
     * parts of the input do not modify the initial configuration while
     * others do.
     */
    public void read(ModStrategy mod) throws ProofInputException {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
	Iterator specsIter = initConfig.getProofEnv().getSpecs();
	Contract spec;
	Taclet specTac;
	SetOfTaclet sOfT;
	Iterator it;
	ContractSet specSet;
	while (specsIter.hasNext()) {
	    specSet = (ContractSet) specsIter.next();
	    it = specSet.iterator();
	    while (it.hasNext()) {
		spec = (Contract) it.next();	   
		specTac = spec.getLemmaAndRegister();
		if (specTac != null) {
		    sOfT = initConfig.getTaclets();
		    sOfT = sOfT.add(specTac);
		    initConfig.setTaclets(sOfT);
		}
	    }
	}		    
    }


    /** returns the name of this input
     */
    public String name() {
	return name;
    }

    /** returns the total numbers of chars that can be read in this input.
     */
    public int getNumberOfChars() {
	return 0;
    }

}
