// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.ProofAggregate;
import de.uka.ilkd.key.proof.mgt.Contract;
import de.uka.ilkd.key.proof.mgt.Contractable;
import de.uka.ilkd.key.proof.mgt.ProofEnvironment;
import de.uka.ilkd.key.logic.SetOfChoice;
import de.uka.ilkd.key.logic.NamespaceSet;
import de.uka.ilkd.key.logic.Choice;
import de.uka.ilkd.key.collection.SetOfString;
import java.util.HashMap;
import de.uka.ilkd.key.proof.SetOfProof;

/**
 * Represents something that produces an input proof obligation for the 
 * prover. A .key file or a proof obligation generated from a CASE tool may 
 * be instances. An instance can be called to set an initial configuration,
 * that is modified during reading the input (using <code>setConfig</code>), 
 * to read the Java model 
 * belonging to the input (using <code>readJava</code>), to read the rest 
 * of the input (using <code>read</code>), and to get the read proof 
 * obligation (using <code>getProblemTerm</code>). During reading the input
 * given initial configuration may become modified. The modification
 * can be controlled by a modification strategy.
 */
public interface ProofOblInput {

    /** returns the proof obligation term as result of the proof obligation
     * input. If there is still no input available because nothing has
     * been read yet null is returned.
     */
    ProofAggregate getPO();

    /** starts reading the input and modifies the InitConfig of this 
     * object with respect to the given modification
     * strategy. 
     */
    void read(ModStrategy mod) throws ProofInputException;

    void readProblem(ModStrategy mod) throws ProofInputException;

    boolean askUserForEnvironment();

    /** reads the Java model that belongs to this input.
     */
    String readJava() throws ProofInputException;

    /** returns the path to the Java model.
     */
    String getJavaPath() throws ProofInputException;

    /** set the initial configuration used to read an input. It may become
     * modified during reading depending on the modification strategy used
     * for reading.
     */
    void setConfig(InitConfig i);

    void readSpecs();

    SetOfChoice readActivatedChoices() throws ProofInputException;
    
    /** reads the include section and returns an Includes object.  
     */
    Includes readIncludes() throws ProofInputException;
    
    /** returns the name of the proof obligation input.
     */
    String name();

    Contractable[] getObjectOfContract();

    boolean initContract(Contract ct);

    void startProtocol();
}
