// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/** this class inherits from TacletGoalTemplate. It is used if there is a
 * replacewith in the ruleGoals that replaces a sequent with a
 * sequent. The replacewith for terms/formulae is realized in another
 * class calles RewriteTacletGoalTemplate.
*/
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.*;

public class AntecSuccTacletGoalTemplate extends TacletGoalTemplate{
    /** sequent that replaces another one */
    private Sequent replacewith=Sequent.EMPTY_SEQUENT;

    /** creates new Goaldescription 
     *@param addedSeq new Sequent to be added
     *@param addedRules ListOfTaclet contains the new allowed rules
     * at this branch 
     *@param ruleVars ListOfRulevariable contains used rulevariables 
     *@param replacewith the Sequent that replaces another one
     */
    public AntecSuccTacletGoalTemplate(Sequent addedSeq,
			   ListOfTaclet addedRules,
			   ListOfRulevariable ruleVars,
			   Sequent replacewith,
			   SetOfSchemaVariable pvs) {
	super(addedSeq, addedRules, ruleVars, pvs);
	TacletBuilder.checkContainsFreeVarSV(replacewith, "replacewith sequent");
	this.replacewith = replacewith;
    }

    public AntecSuccTacletGoalTemplate(Sequent addedSeq,
				     ListOfTaclet addedRules,
				     ListOfRulevariable ruleVars,
				     Sequent replacewith) {
	this(addedSeq, addedRules, ruleVars, replacewith,
	     SetAsListOfSchemaVariable.EMPTY_SET);
    }

    /** a Taclet may replace a Sequent by another. The new Sequent is returned.
     * this Sequent.
     * @return Sequent being paramter in the rule goal replacewith(Seq)
     */

    public Sequent replaceWith() {
	return replacewith;
    }

    /**
     * @return Sequent being paramter in the rule goal replacewith(Seq)
     */
    Object replaceWithExpressionAsObject() {
	return replacewith;
    }

    /**
     * returns true iff the given schema variable is 
     * bound in the sequent part of
     * this goal sequent or in an added rule or in the replacewith sequent.
     * @param v the bound variable to be searched
     */
    boolean varIsBound(SchemaVariable v) {
	if (!v.isVariableSV()) {
	    return false;
	}
	return (replaceWith().varIsBound((SortedSchemaVariable)v) 
		|| super.varIsBound(v));
    }

    /**
     * returns pairs of variables (x,y) where y is bound in this
     * TacletGoalTemplate and x 
     * occurs in the scope of y; (y,y) is part of the set for all bound ys
     * with no x bound in the scope of y. x is a LogicVariableRepresentative 
     * while y is a QuantifiableVariable (as it is bound).
     * @return a set of variable pairs as specified above
     */
    protected SetOfInScopePair getVarInScope() {
	return replaceWith().getVarInScope().union(super.getVarInScope());
    }

    public boolean equals(Object o) {
	if ( ! ( o instanceof AntecSuccTacletGoalTemplate ) )
	    return false;
	AntecSuccTacletGoalTemplate other = (AntecSuccTacletGoalTemplate) o;

	return super.equals(other)
	    && replacewith.equals(other.replacewith);
    }
    
    public int hashCode(){
    	int result = 17;
    	result = 37 * result + super.hashCode();
    	result = 37 * result + replacewith.hashCode();
    	return result;
    }

    
    /** toString */
    public String toString() {
	String result=super.toString();
	result+="\\replacewith("+replaceWith()+") ";       
	return result;
    }

}
