// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.rule.conditions;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.SVSubstitute;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.SortedSchemaVariable;
import de.uka.ilkd.key.rule.VariableConditionAdapter;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.rule.metaconstruct.arith.Monomial;

/**
 * Check whether one monomial <code>b</code> divides another monomial
 * <code>a</code>.
 */
public class MonomialsDivideCondition extends VariableConditionAdapter {

    private final SchemaVariable a, b;
    
    public MonomialsDivideCondition (SchemaVariable a, SchemaVariable b) {
        this.a = a;
        this.b = b;
        if ( !a.isTermSV ()
             || !b.isTermSV ()
             || !"int".equals ( ( (SortedSchemaVariable)a ).sort ()
                                .name ().toString () )
             || !"int".equals ( ( (SortedSchemaVariable)b ).sort ()
                                .name ().toString () ) ) {
            throw new IllegalArgumentException (
                  "Illegal schema variable (only terms of type int are accepted)" );
        }
    }
    
    public boolean check(SchemaVariable var, SVSubstitute term,
                         SVInstantiations matchCond, Services services) {
        final Term instA, instB;
        if (a == var)
            instA = (Term)term;
        else
            instA = (Term)matchCond.getInstantiation ( a );

        if (b == var)
            instB = (Term)term;
        else
            instB = (Term)matchCond.getInstantiation ( b );

        if ( instA == null || instB == null ) return true;

        final Monomial mA = Monomial.create ( instA, services );
        final Monomial mB = Monomial.create ( instB, services );
        
        return mB.divides ( mA );
    }

    public String toString () {
        return "\\monomialsDivide (" + a + ", "+ b + ")";
    }

}
