// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.rule.conditions;


import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermOrdering;
import de.uka.ilkd.key.logic.op.SchemaVariable;

/**
 * Condition for heuristic application: Compare the instantiations of two termSV
 * or formulaSV, using a term ordering. The condition accepts an instantiation
 * iff the instantiation of <code>var0</code> is not strictly smaller than the
 * instantiation of <code>var1</code>
 */
public class NotSmallerThanOrderingCondition extends AbstractOrderingCondition {

    public NotSmallerThanOrderingCondition(SchemaVariable p_var0,
                                           SchemaVariable p_var1) {
        super ( p_var0, p_var1 );
    }

    protected boolean checkInstantiations(Term t0, Term t1, TermOrdering p_ordering) {
        return p_ordering.compare ( t0, t1 ) >= 0;
    }

    public String toString () {
	return "\\notSmallerThan(" + getVar0() + ", " + getVar1() + ")";
    }

}
