// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.rule.MatchConditions;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/** this class implements the interface for
 * MetaAdderators. MetaAdderators are used to do complex term
 * transformation when applying a taclet. Often these transformation
 * caanot be described with the taclet scheme (or trying to do so would
 * result in a huge number of rules)
 */
public abstract class MetaField extends AbstractMetaOperator {

    public MetaField(String name, boolean isStatic) {
	super(new Name(name), isStatic ? 0 : 1 );
    }

    public MetaField(String name) {
	this(name, false);
    }

    public boolean isRigid(Term t) {
	return false;
    }

    /**
     * checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.  
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	// a meta operator accepts almost everything
	return term.arity() == arity();
    }


    /**
     *  does not match anything
     *  (non-Javadoc)
     * @see de.uka.ilkd.key.logic.op.Operator#match(de.uka.ilkd.key.logic.op.Operator, de.uka.ilkd.key.rule.MatchConditions, de.uka.ilkd.key.java.Services)
     */
    public MatchConditions match(Operator op, MatchConditions mc,
            Services services) {
        return null;
    }
    
    /** calculates the resulting term. */
    public abstract Term calculate(Term term, SVInstantiations svInst,
			  Services services);

}
