// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.TacletApp;

/**
 * Feature for excluding that a certain schema variable has a certain
 * top-level symbol (we just use strings for identifying the schema variable
 * and the term symbol). This feature returns zero if and only if the schema
 * variable <code>schemaVar</code> is not instantiated or the instantiation of
 * <code>schemaVar</code> does not have the top symbol <code>topSymbol</code>.
 */
public class ExcludeTopSymbolFeature extends BinaryTacletAppFeature {

    private final Name schemaVar;
    private final Name topSymbol;
    
    private ExcludeTopSymbolFeature(final String schemaVar,
                                    final String topSymbol) {
        this.schemaVar = new Name ( schemaVar );
        this.topSymbol = new Name ( topSymbol );
    }

    protected boolean filter(TacletApp app, PosInOccurrence pos, Goal goal) {
        final Object instObj = app.instantiations ().lookupValue ( schemaVar );
        if ( ! ( instObj instanceof Term ) ) return true;
        	
        final Term inst = (Term)instObj;
        return !topSymbol.equals ( inst.op ().name () );
    }

    public static Feature create(String schemaVar, String topSymbol) {
        return new ExcludeTopSymbolFeature ( schemaVar, topSymbol );
    }
    
}
