// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.RuleApp;

/**
 * This feature returns zero if and only if the focus of the given rule
 * application does not exist, is top-level or the symbol immediately above the
 * focus is not <code>badSymbol</code>. (we just use strings for identifying
 * the evil symbol)
 */
public class NotDirectlyBelowSymbolFeature extends BinaryFeature {

    private final Name badSymbol;

    private NotDirectlyBelowSymbolFeature(final String badSymbol) {
        this.badSymbol = new Name ( badSymbol );
    }

    public static Feature create(String badSymbol) {
        return new NotDirectlyBelowSymbolFeature ( badSymbol );
    }
    
    protected boolean filter(RuleApp app, PosInOccurrence pos, Goal goal) {
        if ( pos == null ) return true;
        if ( pos.isTopLevel () ) return true;
        return !badSymbol.equals ( pos.up ().subTerm ().op ().name () );
    }

}
