// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.strategy.feature;

import java.util.HashMap;
import java.util.Map;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.IteratorOfRuleSet;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.rule.RuleSet;
import de.uka.ilkd.key.rule.TacletApp;
import de.uka.ilkd.key.strategy.LongRuleAppCost;
import de.uka.ilkd.key.strategy.RuleAppCost;


public class RuleSetDispatchFeature implements Feature {

    private final Map rulesetToFeature = new HashMap ();
    
    private RuleSetDispatchFeature() {}
    
    public static RuleSetDispatchFeature create() {
        return new RuleSetDispatchFeature ();
    }
    
    public RuleAppCost compute(RuleApp app, PosInOccurrence pos, Goal goal) {
        if ( ! ( app instanceof TacletApp ) ) return LongRuleAppCost.ZERO_COST;

        RuleAppCost res = LongRuleAppCost.ZERO_COST;
        final IteratorOfRuleSet it = ( (TacletApp)app ).taclet ().ruleSets ();
        while ( it.hasNext () ) {
            final Feature partialF = (Feature)rulesetToFeature.get ( it.next () );
            if ( partialF != null )
                    res = res.add ( partialF.compute ( app, pos, goal ) );
        }

        return res;
    }

    public void add(RuleSet ruleSet, Feature f) {
        Feature combinedF = (Feature)rulesetToFeature.get ( ruleSet );
        if ( combinedF == null )
            combinedF = f;
        else
            combinedF = BinarySumFeature.createSum ( combinedF, f );

        rulesetToFeature.put ( ruleSet, combinedF );
    }
    
}
